; Based on Iain McCurdy's 2015 Grain3FilePlayer.csd
; Revised/expanded by Jon Nelson, 2020

; File player based around the granular synthesis opcode, 'grain3'.
; A second voice can be activated (basically another parallel granular synthesiser) with parameter variations of density, transposition, pointer location (Phs) and delay.
; Two modes of playback are available: manual pointer and speed
; The pointer and grain density can also be modulated by clicking and dragging on the waveform view.
;  * This will also start and stop the grain producing instrument.
;  * In click-and-drag mode mouse X position equates to pointer position and mouse Y position equates to grain density. 
; If played from the MIDI keyboard, note number translates to 'Transposition' and key velocity translates to amplitude for the grain stream for that note.

; In 'Pointer' mode pointer position is controlled by the long 'Manual' slider with an optional amount of randomisation determined ny the 'Phs.Mod' slider.  

; Selecting 'Speed' pointer mode bring up some additional controls:
; Speed		-	speed ratio
; Freeze	-	freezes the pointer at its present locations 
; Range		-	ratio of the full sound file duration that will be played back. 1=the_entire_file, 0.5=half_the_file, etc. 
; Shape		-	shape of playback function: 	'Phasor' looping in a single direction
;							'Tri' back and forth looping
; The 'Manual' control functions as an pointer offset when using 'Speed' pointer mode

; Size		-	size of the grains
; Density	-	grains per second
; Transpose	-	transposition in semitones
; Window	-	window shape that envelopes each individual grain

; --Randomisation--
; Trans.Mod.	-	randomisation of transposition (in octaves)
; Ptr.Mod.	-	randomisation of pointer position
; Dens Mod.	-	randomisation of grain density
; Size Mod.	-	randomisation of grain size

; --Density LFO--
; Density	-	depth of LFO modulation of grain density
; Amplitude	-	depth of LFO modulation of grain amplitude
; Size		-	depth of LFO modulation of grain size
; Rate		-	rate of LFO modulation of grain density

; --Voice 2--
; Dens.Ratio	-	ratio of grain density of voice 2 with respect to the main voice (also adjustable using the adjacent number box for precise value input)
; Ptr.Diff.	-	pointer position offset of voice 2 with respect to the main voice (also adjustable using the adjacent number box for precise value input)
; Trans.Diff.	-	transposition offset of voice 2 with respect to the main voice (also adjustable using the adjacent number box for precise value input)
; Delay		-	a delay applied to voice 2 which is defined as a ratio of the gap between grains (therefore delay time will be inversely proportional to garin density)
;			 This is a little like a phase offset for voice 2 with respect to that of the main voice.
;			 When using this control 'Dens.Ratio' should be '1' otherwise continuous temporally shifting between the grains of voice 2 and the main voice will be occurring anyway.

; --Envelope--
; Attack	-	amplitude envelope attack time for the envelope applied to complete notes
; Release	-	amplitude envelope release time for the envelope applied to complete notes

; --Control--
; MIDI Ref.	-	MIDI note that represent unison (no transposition) for when using the MIDI keyboard
; Level	-	output amplitude control


<Cabbage>
#define PLANT colour(76,130,180,155) outlinecolour(0,0,0,255) outlinethickness(2)
#define BASIC colour(56, 150, 180, 255) trackercolour(56,255,182,255) outlinecolour(17,37,37,155)
#define VBASIC colour("Gainsboro") trackercolour(56,255,182,255) outlinecolour(17,37,37,155)
#define FTABLES tablegridcolour("Silver") tablecolour:0(20,20,20,245) tablebackgroundcolour("Silver")
#define BUTTONS colour:0(79,79,79,105) colour:1("MediumTurquoise") colour:0("DarkSlateGrey")
#define MIDI colour:0(96,150,170,255) colour:1("Grey") value(0) latched(0)
#define MIDIX colour:0(96,150,170,255) colour:1("red") value(0) latched(0) 
#define FONT fontcolour("Gainsboro") align("left")
#define RSliderStyle # colour(56, 150, 180, 255) trackercolour("MediumTurquoise") textcolour("Gainsboro") valuetextbox(1) textboxcolour("black") fontcolour("White")#

form caption("Scrubber") size(1025,1050), colour(76,130,180,155), pluginid("Scrubber"), guirefresh(16) style("legacy")	; guirefresh value adjusts how often the cabbage valuators are read. This can affect the usability of parameters used in realtime gesturing. style("legacy")

image       bounds(0,0,1025,400), colour(76,130,180,155), outlinecolour("Gainsboro"), shape("sharp"), line(3)	; main panel colouration    
soundfiler  bounds(5,5,1015,375), channel("beg","len"), identchannel("filer1"),  colour(0, 255, 255, 255), fontcolour(160, 160, 160, 255), 
label bounds(7,5,560,14), text(""), align(left), colour(0,0,0,0), fontcolour(200,200,200), identchannel("stringbox")
hslider    bounds(0,380,1015,20), channel("phs"),   range( 0,1,0,1,0.0001), $RSliderStyle

filebutton bounds(5,405,90,25) text("Load Sound File") fontcolour("Gainsboro") channel("filename") shape("ellipse")
label       bounds(10,435,75,13) text("Envelope") fontcolour("Gainsboro")
combobox    bounds(5,450,90,15) channel("wfn") items("Hanning","Perc 1","Perc 2","Perc 3","Gate","Rev Perc 1","Rev Perc 2","Rev Perc 3"), value(1),fontcolour("Gainsboro")

checkbox bounds(99,405,125,16) channel("PlayStop") text("  Play/Stop") fontcolour("Gainsboro")
label bounds(100,422,145,9) text("or right-click/drag") fontcolour("Gainsboro") align("left")

combobox    bounds(100,433,75,15), channel("PhsMode"), items("Scrubber", "Loop"), value(2),fontcolour("Gainsboro")
combobox    bounds(100,449,75,15), channel("shape"), items("forward", "<--->"), value(1),fontcolour("Gainsboro"), identchannel("shapeID"), visible(0)
button bounds(183,410,30,15) text("Speed") channel("mapspd") $MIDI
button bounds(213,410,9,15) text("") channel("mapspdX") $MIDIX
rslider     bounds(173,430,60,50), channel("spd"),     range( -2.00, 2.00, 1) identchannel("spdID"), visible(0), $RSliderStyle
button      bounds(100,465,77,17), channel("freeze"),  colour(  0,  0,  0), text("Freeze","Freeze"), colour:1("MediumTurquoise") colour:0("Black") fontcolour:0("Gainsboro"), fontcolour:1("Gainsboro"), identchannel("freezeID"), visible(0)
label bounds(255,405,80,13) text("Loop Range") $FONT
hrange bounds(225,430,143,30) channel("start","end") range(0,1,0:1,1,.000001) textbox(1) caption("Loop Range") $VBASIC
nslider bounds(230,464,50,16) channel("start") range(0,1,0,1,.000001)
nslider bounds(320,464,50,16) channel("end") range(0,1,1,1,.000001)

button bounds(375,410,30,15) text("Dur") channel("mapdur") $MIDI
button bounds(405,410,9,15) text("") channel("mapdurX") $MIDIX
rslider     bounds(365,430,60,50) channel("dur") range(0.01,5.00,0.05,0.5,0.001) $RSliderStyle
button bounds(430,410,30,15) text("Rate") channel("mapdens") $MIDI
button bounds(460,410,9,15) text("") channel("mapdensX") $MIDIX
rslider     bounds(420,430,60,50) channel("dens") range( 0.2, 500,  20, 0.5) $RSliderStyle
button bounds(485,410,30,15) text("Pitch") channel("mappch") $MIDI
button bounds(515,410,9,15) text("") channel("mappchX") $MIDIX
rslider     bounds(475,430,60,50) channel("pch") range(-2,2,1,1,0.001) $RSliderStyle
label bounds(555,405,100,13) text("Random Panning") $FONT
label bounds(545,420,25,10) text("rate") $FONT
rslider bounds(530,430,50,50) channel("prate") range(0,50,20,.75,.01) $RSliderStyle
label bounds(595,420,25,10) text("rand") $FONT
rslider bounds(580,430,50,50) channel("prand") range(0,.99,.25,1,.01) $RSliderStyle
label bounds(638,420,35,10) text("spread") $FONT
rslider bounds(630,430,50,50) channel("pspread") range(0,1,.75,1,.01) $RSliderStyle

button bounds(950,405,65,18), channel("record") text("Record","Stop") colour("red") fontcolour("white")
combobox bounds(950,427,65,13) channel("recch") value(2) text("mono","stereo")
combobox bounds(950,443,65,13) channel("bit") value(1) text("16-bit","24-bit","32-bit")
button bounds(950,461,65,18), channel("play") text("Play") colour("green") fontcolour("white")

image       bounds(680,402,260,80) plant("randomise") $PLANT
{ 
label       bounds(0,2,260,8), text("R  A  N  D  O  M  I  Z  E"), fontcolour("Gainsboro")
button bounds(20,12,30,15) text("Pitch") channel("mapfmd") $MIDI
button bounds(50,12,9,15) text("") channel("mapfmdX") $MIDIX
rslider     bounds(10,27,60,50) channel("fmd") range(0,8,0) $RSliderStyle
button bounds(80,12,30,15) text("SF Loc") channel("mappmd") $MIDI
button bounds(110,12,9,15) text("") channel("mappmdX") $MIDIX
rslider     bounds(70,27,60,50) channel("pmd") range(0,1,0,0.25,0.00001) $RSliderStyle
button bounds(140,12,30,15) text("Rate") channel("mapdensrnd") $MIDI
button bounds(170,12,9,15) text("") channel("mapdensrndX") $MIDIX
rslider     bounds(130,27,60,50) channel("DensRnd") range(0,2,0) $RSliderStyle
button bounds(200,12,30,15) text("Dur") channel("mapsizernd") $MIDI
button bounds(230,12,9,15) text("") channel("mapsizerndX") $MIDIX
rslider     bounds(190,27,60,50) channel("SizeRnd") range(0,8,0) $RSliderStyle
}

image      bounds(5,485,505,80) plant("dual") $PLANT
{ 
label      bounds(0,2,505,8), text("V  O  I  C  E     2"), fontcolour("Gainsboro")
checkbox   bounds(10,30,70,20), channel("DualOnOff"), text("On/Off"), fontcolour("Gainsboro")
button bounds(80,12,50,15) text("Dens.Ratio") channel("mapdensratio") $MIDI
button bounds(130,12,9,15) text("") channel("mapdensratioX") $MIDIX
rslider    bounds(80,27,60,50), channel("DensRatio") range(0.5,2,1,0.64,0.00001)  $RSliderStyle
nslider  bounds(140,25,60,20), channel("DensRatio") range(0.5,2,1,0.64,0.00001) textcolour("white") trackercolour(190,170,130) outlinecolour(100,100,100)
button bounds(200,12,50,15) text("Ptr.Diff.") channel("mapptrdiff") $MIDI
button bounds(250,12,9,15) text("") channel("mapptrdiffX") $MIDIX
rslider    bounds(200,27,60,50), channel("PtrDiff"),   range(-1,1,0,1,0.00001) $RSliderStyle
nslider  bounds(260,25,60,20), channel("PtrDiff"),   range(-1,1,0,1,0.00001), textcolour("white"), trackercolour(190,170,130), outlinecolour(100,100,100)
button bounds(320,12,50,15) text("Trans.Diff.") channel("maptransdiff") $MIDI
button bounds(370,12,9,15) text("") channel("maptransdiffX") $MIDIX
rslider    bounds(320,27,60,50) channel("TransDiff") range(-2,2,0,1,0.00001) $RSliderStyle
nslider  bounds(380,25,60,20) channel("TransDiff") range(-2,2,0,1,0.00001), textcolour("white"), trackercolour(190,170,130), outlinecolour(100,100,100)
button bounds(440,12,50,15) text("Delay") channel("mapdelay") $MIDI
button bounds(490,12,9,15) text("") channel("mapdelayX") $MIDIX
rslider    bounds(440,27,60,50) channel("Delay") range(0,1,0,1,0.00001) $RSliderStyle
}

image      bounds(510,485,250,80) plant("LFO") $PLANT
{ 
label      bounds(0,2,250,8), text("L  F  O"), fontcolour("Gainsboro")
button bounds(10,12,40,15) text("Density") channel("mapdenslfodep") $MIDI
button bounds(50,12,9,15) text("") channel("mapdenslfodepX") $MIDIX
rslider    bounds(5,27,60,50) channel("DensLFODep") range(-4,4,0,1,0.001) $RSliderStyle
button bounds(70,12,40,15) text("Amp") channel("mapamplfodep") $MIDI
button bounds(110,12,9,15) text("") channel("mapamplfodepX") $MIDIX
rslider    bounds(65,27,60,50) channel("AmpLFODep") range(-1, 1, 0, 1, 0.001) $RSliderStyle
button bounds(130,12,40,15) text("Dur") channel("mapsizelfodep") $MIDI
button bounds(170,12,9,15) text("") channel("mapsizelfodepX") $MIDIX
rslider    bounds(125,27,60,50) channel("SizeLFODep") range(-2, 2, 0, 1, 0.001) $RSliderStyle
button bounds(190,12,40,15) text("Rate") channel("maplforte") $MIDI
button bounds(230,12,9,15) text("") channel("maplforteX") $MIDIX
rslider    bounds(185,27,60,50) channel("LFORte") range(0.01, 8, 0.1, 0.5, 0.001) $RSliderStyle
}

image      bounds(760,485,130,80) plant("envelope") $PLANT
{ 
label      bounds(0,2,140,8), text("E   N   V   E   L   O   P   E"), fontcolour("Gainsboro")
button bounds(10,12,40,15) text("Attack") channel("mapatttim") $MIDI
button bounds(50,12,9,15) text("") channel("mapatttimX") $MIDIX
rslider    bounds(5,27,60,50) channel("AttTim") range(0, 5, 0, 0.5, 0.001) $RSliderStyle
button bounds(70,12,40,15) text("Release") channel("mapreltim") $MIDI
button bounds(110,12,9,15) text("") channel("mapreltimX") $MIDIX
rslider    bounds(65,27,60,50) channel("RelTim") range(0.01, 5, 0.05, 0.5, 0.001) $RSliderStyle
}

image      bounds(890,485,130,80) plant("control") $PLANT
{ 
label      bounds(0,2,140,8) text("C   O   N   T   R   O   L") fontcolour("Gainsboro")
button bounds(5,12,50,15) text("MIDI Ref.") channel("mapmidiref") $MIDI
button bounds(55,12,9,15) text("") channel("mapmidirefX") $MIDIX
rslider    bounds(5,27,60,50) channel("MidiRef") range(0,127,60,1,1) $RSliderStyle
button bounds(65,12,50,15) text("Level") channel("maplevel") $MIDI
button bounds(115,12,9,15) text("") channel("maplevelX") $MIDIX
rslider    bounds(65,27,60,50) channel("level") range(0,3.00,0.7,0.5,0.001) $RSliderStyle
}

image bounds(5,565,185,300) plant("FFT") $PLANT
{
button bounds(13,5,40,18) channel("shifton") text("Shift") value((0) $BUTTONS
combobox   bounds(10,25,50,12), channel("shiftsize"), items("32768", "16384", "8192", "4096", "2048", "1024", "512", "256", "128", "64", "32", "16", "8", "4"), value(6) $FONT
button bounds(15,40,30,15) text("Shift") channel("mapshift") $MIDI
button bounds(45,40,9,15) text("") channel("mapshiftX") $MIDIX
rslider bounds(0,55,70,45) channel("shift") range(-1,1,0,1,.0001) $RSliderStyle
button bounds(15,105,30,15) text("Lowest") channel("maplowest") $MIDI
button bounds(45,105,9,15) text("") channel("maplowestX") $MIDIX
rslider bounds(0,120,70,45) channel("lowest") range(0,1,0,.5,.0001) $RSliderStyle
button bounds(15,235,30,15) text("Mix") channel("mapshiftmix") $MIDI
button bounds(45,235,9,15) text("") channel("mapshiftmixX") $MIDIX
rslider bounds(0,250,70,45) channel("shiftmix") range(0,1,.5,1,.01) $RSliderStyle

button bounds(73,5,40,18) channel("warpon") text("Warp") value(0) $BUTTONS
combobox   bounds(70,25,50,13), channel("warpsize"), items("32768", "16384", "8192", "4096", "2048", "1024", "512", "256", "128", "64", "32", "16", "8", "4"), value(6) $FONT
button bounds(75,40,30,15) text("Scale") channel("mapenvscale") $MIDI
button bounds(105,40,9,15) text("") channel("mapenvscaleX") $MIDIX
rslider bounds(60,55,70,45) channel("envscale") range(.1,10,1,.5,.0001) $RSliderStyle
button bounds(75,105,30,15) text("Shift") channel("mapenvshift") $MIDI
button bounds(105,105,9,15) text("") channel("mapenvshiftX") $MIDIX
rslider bounds(60,120,70,45) channel("envshift") range(-1,1,0,1,.0001) $RSliderStyle
button bounds(75,170,30,15) text("Lowest") channel("mapenvlowest") $MIDI
button bounds(105,170,9,15) text("") channel("mapenvlowestX") $MIDIX
rslider bounds(60,185,70,45) channel("envlowest") range(0,1,0,.5,.0001) $RSliderStyle
button bounds(75,235,30,15) text("Mix") channel("mapwarpmix") $MIDI
button bounds(105,235,9,15) text("") channel("mapwarpmixX") $MIDIX
rslider bounds(60,250,70,45) channel("warpmix") range(0,1,.5,1,.01) $RSliderStyle

button bounds(133,5,40,18) channel("smoothon") text("Smooth") value(0) $BUTTONS
combobox   bounds(130,25,50,13), channel("smoothsize"), items("32768", "16384", "8192", "4096", "2048", "1024", "512", "256", "128", "64", "32", "16", "8", "4"), value(6) $FONT
button bounds(135,40,30,15) text("Amp") channel("mapampsmooth") $MIDI
button bounds(165,40,9,15) text("") channel("mapampsmoothX") $MIDIX
rslider bounds(120,55,70,45) channel("ampsmooth") range(0,1,0,.5,.0001) $RSliderStyle
button bounds(135,105,30,15) text("Freq") channel("mapfreqsmooth") $MIDI
button bounds(165,105,9,15) text("") channel("mapfreqsmoothX") $MIDIX
rslider bounds(120,120,70,45) channel("freqsmooth") range(0,1,0,.5,.0001) $RSliderStyle
button bounds(135,235,30,15) text("Mix") channel("mapsmoothmix") $MIDI
button bounds(165,235,9,15) text("") channel("mapsmoothmixX") $MIDIX
rslider bounds(120,250,70,45) channel("smoothmix") range(0,1,.5,1,.01) $RSliderStyle
}

image bounds(190,565,375,300) plant("Arpeggiator") $PLANT
{
button bounds(10,5,60,18) channel("arpon") text("Arpeggios") value(0) $BUTTONS
combobox   bounds(15,25,50,13), channel("arpsize"), items("32768", "16384", "8192", "4096", "2048", "1024", "512", "256", "128", "64", "32", "16", "8", "4"), value(6) $FONT
button bounds(20,40,30,15) text("Bin") channel("maparpbin1") $MIDI
button bounds(50,40,9,15) text("") channel("maparpbin1X") $MIDIX
rslider bounds(0,55,80,45) channel("arpbin1") range(0,1,.1,.5,.0001) $RSliderStyle
button bounds(15,105,40,15) text("B Damp") channel("maparpdepth1") $MIDI
button bounds(55,105,9,15) text("") channel("maparpdepth1X") $MIDIX
rslider bounds(0,120,80,45) channel("arpdepth1") range(0,100,.75,.5,.01) $RSliderStyle
button bounds(15,170,40,15) text("B Gain") channel("maparpgain1") $MIDI
button bounds(55,170,9,15) text("") channel("maparpgain1X") $MIDIX
rslider bounds(0,185,80,45) channel("arpgain1") range(0,100,2,.5,.01) $RSliderStyle
button bounds(20,235,30,15) text("Amp") channel("maparpamp1") $MIDI
button bounds(50,235,9,15) text("") channel("maparpamp1X") $MIDIX
rslider bounds(0,250,80,45) channel("arpamp1") range(0,1,.5,1,.01) $RSliderStyle
combobox bounds(75,20,50,13) channel("arpshape1") text("sine", "triangle", "square", "rampup", "rampdown", "pulse", "S&H", "rand")
button bounds(75,40,40,15) text("V Width") channel("maparpwidth1") $MIDI
button bounds(115,40,9,15) text("") channel("maparpwidth1X") $MIDIX
rslider bounds(60,55,80,45) channel("arpwidth1") range(0,1,.1,.5,.0001) $RSliderStyle
button bounds(75,105,40,15) text("V Speed") channel("maparpspeed1") $MIDI
button bounds(115,105,9,15) text("") channel("maparpspeed1X") $MIDIX
rslider bounds(60,120,80,45) channel("arpspeed1") range(0,100,0,.5,.01) $RSliderStyle
button bounds(75,170,40,15) text("V Rand") channel("maparprand1") $MIDI
button bounds(115,170,9,15) text("") channel("maparprand1X") $MIDIX
rslider bounds(60,185,80,45) channel("arprand1") range(0,1,0,1,.01) $RSliderStyle

button bounds(140,40,30,15) text("Bin") channel("maparpbin2") $MIDI
button bounds(170,40,9,15) text("") channel("maparpbin2X") $MIDIX
rslider bounds(120,55,80,45) channel("arpbin2") range(0,1,.2,.5,.0001) $RSliderStyle
button bounds(135,105,40,15) text("B Damp") channel("maparpdepth2") $MIDI
button bounds(175,105,9,15) text("") channel("maparpdepth2X") $MIDIX
rslider bounds(120,120,80,45) channel("arpdepth2") range(0,100,.75,.5,.01) $RSliderStyle
button bounds(135,170,40,15) text("B Gain") channel("maparpgain2") $MIDI
button bounds(175,170,9,15) text("") channel("maparpgain2X") $MIDIX
rslider bounds(120,185,80,45) channel("arpgain2") range(0,100,2,.5,.01) $RSliderStyle
button bounds(140,235,30,15) text("Amp") channel("maparpamp2") $MIDI
button bounds(170,235,9,15) text("") channel("maparpamp2X") $MIDIX
rslider bounds(120,250,80,45) channel("arpamp2") range(0,1,.5,1,.01) $RSliderStyle
combobox bounds(195,20,50,13) channel("arpshape2") text("sine", "triangle", "square", "rampup", "rampdown", "pulse", "S&H", "rand")
button bounds(195,40,40,15) text("V Width") channel("maparpwidth2") $MIDI
button bounds(235,40,9,15) text("") channel("maparpwidth2X") $MIDIX
rslider bounds(180,55,80,45) channel("arpwidth2") range(0,1,.1,.5,.0001) $RSliderStyle
button bounds(195,105,40,15) text("V Speed") channel("maparpspeed2") $MIDI
button bounds(235,105,9,15) text("") channel("maparpspeed2X") $MIDIX
rslider bounds(180,120,80,45) channel("arpspeed2") range(0,100,0,.5,.01) $RSliderStyle
button bounds(195,170,40,15) text("V Rand") channel("maparprand2") $MIDI
button bounds(235,170,9,15) text("") channel("maparprand2X") $MIDIX
rslider bounds(180,185,80,45) channel("arprand2") range(0,1,0,1,.01) $RSliderStyle

button bounds(260,40,30,15) text("Bin") channel("maparpbin3") $MIDI
button bounds(290,40,9,15) text("") channel("maparpbin3X") $MIDIX
rslider bounds(240,55,80,45) channel("arpbin3") range(0,1,.3,.5,.0001) $RSliderStyle
button bounds(255,105,40,15) text("B Damp") channel("maparpdepth3") $MIDI
button bounds(295,105,9,15) text("") channel("maparpdepth3X") $MIDIX
rslider bounds(240,120,80,45) channel("arpdepth3") range(0,100,.75,.5,.01) $RSliderStyle
button bounds(255,170,40,15) text("B Gain") channel("maparpgain3") $MIDI
button bounds(295,170,9,15) text("") channel("maparpgain3X") $MIDIX
rslider bounds(240,185,80,45) channel("arpgain3") range(0,100,2,.5,.01) $RSliderStyle
button bounds(260,235,30,15) text("Amp") channel("maparpamp3") $MIDI
button bounds(290,235,9,15) text("") channel("maparpamp3X") $MIDIX
rslider bounds(240,250,80,45) channel("arpamp3") range(0,1,.5,1,.01) $RSliderStyle
combobox bounds(315,20,50,13) channel("arpshape3") text("sine", "triangle", "square", "rampup", "rampdown", "pulse", "S&H", "rand")
button bounds(315,40,40,15) text("V Width") channel("maparpwidth3") $MIDI
button bounds(355,40,9,15) text("") channel("maparpwidth3X") $MIDIX
rslider bounds(300,55,80,45) channel("arpwidth3") range(0,1,.1,.5,.0001) $RSliderStyle
button bounds(315,105,40,15) text("V Speed") channel("maparpspeed3") $MIDI
button bounds(355,105,9,15) text("") channel("maparpspeed3X") $MIDIX
rslider bounds(300,120,80,45) channel("arpspeed3") range(0,100,0,.5,.01) $RSliderStyle
button bounds(315,170,40,15) text("V Rand") channel("maparprand3") $MIDI
button bounds(355,170,9,15) text("") channel("maparprand3X") $MIDIX
rslider bounds(300,185,80,45) channel("arprand3") range(0,1,0,1,.01) $RSliderStyle

button bounds(320,235,30,15) text("Mix") channel("maparpmix") $MIDI
button bounds(350,235,9,15) text("") channel("maparpmixX") $MIDIX
rslider bounds(305,250,70,45) channel("arpmix") range(0,1,.5,1,.01) $RSliderStyle
}

image bounds(565,565,425,175) plant("Transposers") $PLANT
{
button bounds(5,5,75,18) channel("ton") text("Transposer", "Transposer") $BUTTONS
combobox   bounds(100,5,50,13), channel("Tsize"), items("32768", "16384", "8192", "4096", "2048", "1024", "512"), value(6) $FONT

combobox bounds(200,5,80,13) channel("Tpresets") text("Fat","Fatter","Warble","5ths","4ths","M3rds","Mi3rds","Cluster 1","Cluster 2","Whole Tones","Stacked 5ths","Stacked 4ths","Stacked M3rds","Stacked Mi3rds","8ves","Big 5ths","Major","Minor","V7","M7","M9","Mi7","Sus","V7sus","V7+9","V9","V7+9B")
button bounds(15,44,30,15) text("Port") channel("mapTport") $MIDI
button bounds(45,44,9,15) text("") channel("mapTportX") $MIDIX
rslider bounds(0,60,70,45) channel("Tport") range(0,5,.02,.25,0.01) $RSliderStyle
button bounds(15,107,30,15) text("Mix") channel("mapharmmix") $MIDI
button bounds(45,107,9,15) text("") channel("mapharmmixX") $MIDIX
rslider bounds(0,123,70,45) channel("harmmix") range(0, 1, .5, 1, 0.01) $RSliderStyle

button bounds(83,27,22,15) channel("harmon1") text("Off", "On") value(1) $BUTTONS
button bounds(75,44,30,15) text("Coarse") channel("mapharmon1C") $MIDI
button bounds(105,44,9,15) text("") channel("mapharmon1CX") $MIDIX
rslider bounds(60,60,70,45) channel("harmon1C") range(-24, 24, 0, 1, 1) $RSliderStyle
button bounds(75,107,30,15) text("Fine") channel("mapharmon1F") $MIDI
button bounds(105,107,9,15) text("") channel("mapharmon1FX") $MIDIX
rslider bounds(60,123,70,45) channel("harmon1F") range(-1, 1, .14, 1, 0.01) $RSliderStyle
button bounds(143,27,22,15) channel("harmon2") text("Off", "On") value(1) $BUTTONS
button bounds(135,44,30,15) text("Coarse") channel("mapharmon2C") $MIDI
button bounds(165,44,9,15) text("") channel("mapharmon2CX") $MIDIX
rslider bounds(120,60,70,45) channel("harmon2C") range(-24, 24, 0, 1, 1) $RSliderStyle
button bounds(135,107,30,15) text("Fine") channel("mapharmon2F") $MIDI
button bounds(165,107,9,15) text("") channel("mapharmon2FX") $MIDIX
rslider bounds(120,123,70,45) channel("harmon2F") range(-1, 1, 0-.16, 1, 0.01) $RSliderStyle
button bounds(203,27,22,15) channel("harmon3") text("Off", "On") value(1) $BUTTONS
button bounds(195,44,30,15) text("Coarse") channel("mapharmon3C") $MIDI
button bounds(225,44,9,15) text("") channel("mapharmon3CX") $MIDIX
rslider bounds(180,60,70,45) channel("harmon3C") range(-24, 24, 0, 1, 1) $RSliderStyle
button bounds(195,107,30,15) text("Fine") channel("mapharmon3F") $MIDI
button bounds(225,107,9,15) text("") channel("mapharmon3FX") $MIDIX
rslider bounds(180,123,70,45) channel("harmon3F") range(-1, 1, .09, 1, 0.01) $RSliderStyle
button bounds(263,27,22,15) channel("harmon4") text("Off", "On") value(1) $BUTTONS
button bounds(255,44,30,15) text("Coarse") channel("mapharmon4C") $MIDI
button bounds(285,44,9,15) text("") channel("mapharmon4CX") $MIDIX
rslider bounds(240,60,70,45) channel("harmon4C") range(-24, 24, 0, 1, 1) $RSliderStyle
button bounds(255,107,30,15) text("Fine") channel("mapharmon4F") $MIDI
button bounds(285,107,9,15) text("") channel("mapharmon4FX") $MIDIX
rslider bounds(240,123,70,45) channel("harmon4F") range(-1, 1, 0-.12, 1, 0.01) $RSliderStyle
button bounds(323,27,22,15) channel("harmon5") text("Off", "On") value(1) $BUTTONS
button bounds(315,44,30,15) text("Coarse") channel("mapharmon5C") $MIDI
button bounds(345,44,9,15) text("") channel("mapharmon5CX") $MIDIX
rslider bounds(300,60,70,45) channel("harmon5C") range(-24, 24, 0, 1, 1) $RSliderStyle
button bounds(315,107,30,15) text("Fine") channel("mapharmon5F") $MIDI
button bounds(345,107,9,15) text("") channel("mapharmon5FX") $MIDIX
rslider bounds(300,123,70,45) channel("harmon5F") range(-1, 1, 0.11, 1, 0.01) $RSliderStyle
button bounds(383,27,22,15) channel("harmon6") text("Off", "On") value(1) $BUTTONS
button bounds(375,44,30,15) text("Coarse") channel("mapharmon6C") $MIDI
button bounds(405,44,9,15) text("") channel("mapharmon6CX") $MIDIX
rslider bounds(360,60,70,45) channel("harmon6C") range(-24, 24, 0, 1, 1) $RSliderStyle
button bounds(375,107,30,15) text("Fine") channel("mapharmon6F") $MIDI
button bounds(405,107,9,15) text("") channel("mapharmon6FX") $MIDIX
rslider bounds(360,123,70,45) channel("harmon6F") range(-1, 1, -.05, 1, 0.01) $RSliderStyle
}

;Distortion
image bounds(565,740,135,70) plant("Distort") $PLANT
{
button bounds(45,5,45,18) channel("diston") text("Distort", "Distort") value(0) $BUTTONS
button bounds(5,25,30,15) text("Dist") channel("mapdist") $MIDI
button bounds(35,25,9,15) text("") channel("mapdistX") $MIDIX
rslider bounds(10,40,25,25) channel("dist") range(0, 1, 0, 1, 0.01) $BASIC 
button bounds(47,25,30,15) text("Shape") channel("mapdist2") $MIDI
button bounds(77,25,9,15) text("") channel("mapdist2X") $MIDIX
rslider bounds(52,40,25,25) channel("dist2") range(0, .999, 0, 1, 0.001) $BASIC 
button bounds(89,25,30,15) text("Fold") channel("mapdist3") $MIDI
button bounds(119,25,9,15) text("") channel("mapdist3X") $MIDIX
rslider bounds(94,40,25,25) channel("dist3") range(1, 25, 1, .5, 0.001) $BASIC 
}

;LP Filter
image bounds(700,740,135,90) plant("LP") $PLANT
{
button bounds(43,2,50,17) channel("LPon") text("Lowpass", "Lowpass") value(0) $BUTTONS
combobox bounds(85,20,45,12) channel("LPsel") text("-6 dB","-12 dB","-18 dB","-24 dB")
label bounds(38,50,55,8) text("rate  depth") $FONT
label bounds(97,39,25,8) text("rand") $FONT
button bounds(5,15,27,13) text("Cutoff") channel("mapLPcps") $MIDI
button bounds(32,15,9,13) text("") channel("mapLPcpsX") $MIDIX
rslider bounds(10,26,25,25) channel("LPcps") range(2, 14, 11, 1, .001) $BASIC ;pitch expressed in oct
button bounds(5,50,27,13) text("Res") channel("mapLPres") $MIDI
button bounds(32,50,9,13) text("") channel("mapLPresX") $MIDIX
rslider bounds(10,61,25,25) channel("LPres") range(0.01, .99, 0.1, 1, 0.01) $BASIC
rslider bounds(44,30,20,20) channel("LPcfr") range(0,75,0,.5,.01) $BASIC
rslider bounds(65,30,20,20) channel("LPcfd") range(0,2.5,0,.5,.01) $BASIC
rslider bounds(100,48,20,20) channel("LPrand") range(0,.95,0,1,.01) $BASIC
rslider bounds(44,63,20,20) channel("LPresr") range(0,75,0,.5,.01) $BASIC
rslider bounds(65,63,20,20) channel("LPresd") range(0,2.5,0,.5,.01) $BASIC
}

;Parallel Filters (selectable)
image bounds(835,740,135,195) plant("parallel") $PLANT
{
button bounds(5,2,45,17) channel("paron") text("Filters") value(0) $BUTTONS
combobox bounds(53,3,78,15) channel("vpresets") text("Voice Formants","Soprano ah","Soprano eh","Soprano ih","Soprano oh","Soprano uh","Alto ah","Alto eh","Alto ih","Alto oh","Alto uh","Tenor ah","Tenor eh","Tenor ih","Tenor oh","Tenor uh","Bass ah","Bass eh","Bass ih","Bass oh","Bass uh", )
label bounds(12,167,25,8) text("port") $FONT
rslider bounds(15,175,20,20) channel("fport") range(0,5,.1,.25,0.01) $BASIC 
label bounds(5,20,35,10) text("Select") $FONT
label bounds(36,20,35,10) text("Cutoff") $FONT
label bounds(4,55,55,8) text("rate depth rand") $FONT
label bounds(65,20,35,10) text("Res") $FONT
label bounds(96,20,35,10) text("Amp") $FONT
combobox bounds(5,35,36,12) channel("parsel1") text("LP","BP","HP","off") identchannel("ps1") value(1) 
combobox bounds(5,75,36,12) channel("parsel2") text("LP","BP","HP","off") value(2)
combobox bounds(5,115,36,12) channel("parsel3") text("LP","BP","HP","off") value(2)
combobox bounds(5,155,36,12) channel("parsel4") text("LP","BP","HP","off") value(3)
rslider bounds(41,29,25,25) channel("parF1") range(2,14,8,1,0.001) $BASIC  ;expressing pitch in oct and resonance needs to be between .5-25
rslider bounds(41,69,25,25) channel("parF2") range(2,14,9,1,0.001) $BASIC 
rslider bounds(41,109,25,25) channel("parF3") range(2,14,10,1,0.001) $BASIC 
rslider bounds(41,149,25,25) channel("parF4") range(2,14,11,1,0.001) $BASIC 
rslider bounds(60,49,20,20) channel("parcfr1") range(0,75,0,.5,.01) $BASIC
rslider bounds(60,89,20,20) channel("parcfr2") range(0,75,0,.5,.01) $BASIC
rslider bounds(60,129,20,20) channel("parcfr3") range(0,75,0,.5,.01) $BASIC
rslider bounds(60,169,20,20) channel("parcfr4") range(0,75,0,.5,.01) $BASIC
rslider bounds(80,49,20,20) channel("parcfd1") range(0,2.5,0,.5,.001) $BASIC ;depth calculated in oct as well
rslider bounds(80,89,20,20) channel("parcfd2") range(0,2.5,0,.5,.001) $BASIC
rslider bounds(80,129,20,20) channel("parcfd3") range(0,2.5,0,.5,.001) $BASIC
rslider bounds(80,169,20,20) channel("parcfd4") range(0,2.5,0,.5,.001) $BASIC
rslider bounds(100,49,20,20) channel("parcfrnd1") range(0,.95,0,1,.01) $BASIC
rslider bounds(100,89,20,20) channel("parcfrnd2") range(0,.95,0,1,.01) $BASIC
rslider bounds(100,129,20,20) channel("parcfrnd3") range(0,.95,0,1,.01) $BASIC
rslider bounds(100,169,20,20) channel("parcfrnd4") range(0,.95,0,1,.01) $BASIC
rslider bounds(71,29,25,25) channel("parres1") range(.5,25,1,1,0.01) $BASIC
rslider bounds(71,69,25,25) channel("parres2") range(.5,25,1,1,0.01) $BASIC 
rslider bounds(71,109,25,25) channel("parres3") range(.5,25,1,1,0.01) $BASIC 
rslider bounds(71,149,25,25) channel("parres4") range(.5,25,1,1,0.01) $BASIC
rslider bounds(101,29,25,25) channel("paramp1") range(0,1.5,1,1,0.001) $BASIC 
rslider bounds(101,69,25,25) channel("paramp2") range(0,1.5,1,1,0.001) $BASIC 
rslider bounds(101,109,25,25) channel("paramp3") range(0,1.5,1,1,0.001) $BASIC
rslider bounds(101,149,25,25) channel("paramp4") range(0,1.5,1,1,0.001) $BASIC
}

;phaser
image bounds(5,865,77,100) plant("Phaser") $PLANT
{
button bounds(16,2,45,17) channel("phson") text("Phaser", "Phaser") $BUTTONS
button bounds(10,28,27,15) text("Speed") channel("mapphscps") $MIDI
button bounds(37,28,9,15) text("") channel("mapphscpsX") $MIDIX
rslider bounds(47,23,25,25) channel("phscps") range(0.01, 100, .4, 0.25, 0.01) $BASIC
button bounds(10,51,27,15) text("FB") channel("mapphsFB") $MIDI
button bounds(37,51,9,15) text("") channel("mapphsFBX") $MIDIX
rslider bounds(47,46,25,25) channel("phsFB") range(0, 1, 0.5, 1, 0.01) $BASIC
button bounds(10,74,27,15) text("Q") channel("mapphsQ") $MIDI
button bounds(37,74,9,15) text("") channel("mapphsQX") $MIDIX
rslider bounds(47,70,25,25) channel("phsQ") range(0.01, 100, 1, .25, 0.01) $BASIC
}

;flanger
image bounds(82,865,58,100) plant("Flanger") $PLANT
{
button bounds(8,2,40,17) channel("flon") text("Flanger", "Flanger") $BUTTONS
button bounds(11,25,27,15) text("Speed") channel("mapflcps") $MIDI
button bounds(38,25,9,15) text("") channel("mapflcpsX") $MIDIX
rslider bounds(16,38,25,25) channel("flcps") range(0.01, 100, .05, 0.25, 0.01) $BASIC
button bounds(11,61,27,15) text("FB") channel("mapflFB") $MIDI
button bounds(38,61,9,15) text("") channel("mapflFBX") $MIDIX
rslider bounds(16,74,25,25) channel("flFB") range(0, .99, 0.5, 1, 0.01) $BASIC
}

;Chorus
image bounds(140,865,135,100) plant("Chorus") $PLANT
{
;label bounds(2, 2, 35, 10) text("Chorus") $FONT
button bounds(40,2,45,17) channel("chon") text("Chorus", "Chorus") $BUTTONS
button bounds(5,24,27,15) text("Mix") channel("mapchmix") $MIDI
button bounds(32,24,9,15) text("") channel("mapchmixX") $MIDIX
button bounds(44,24,27,15) text("Rate") channel("mapchcps") $MIDI
button bounds(71,24,9,15) text("") channel("mapchcpsX") $MIDIX
button bounds(83,24,27,15) text("FB") channel("mapchFB") $MIDI
button bounds(110,24,9,15) text("") channel("mapchFBX") $MIDIX
rslider bounds(9, 37, 25, 25) channel("chmix") range(0, 1, .5, 1, 0.01) $BASIC 
rslider bounds(48, 37, 25, 25) channel("chcps") range(.01, 25, .5, .5, 0.001) $BASIC 
rslider bounds(87, 37, 25, 25) channel("chFB") range(0, .999, .7, 1, 0.001) $BASIC 
button bounds(5,60,27,15) text("Depth") channel("mapchdepth") $MIDI
button bounds(32,60,9,15) text("") channel("mapchdepthX") $MIDIX
button bounds(43,60,27,15) text("Delay") channel("mapchdel") $MIDI
button bounds(71,60,9,15) text("") channel("mapchdelX") $MIDIX
button bounds(83,60,27,15) text("Rand") channel("mapchrand") $MIDI
button bounds(110,60,9,15) text("") channel("mapchrandX") $MIDIX
rslider bounds(9, 73, 25, 25) channel("chdepth") range(.1, 50, 1.5, .5, 0.01) $BASIC 
rslider bounds(48, 73, 25, 25) channel("chdel") range(.1, 50, 15, 1, 0.01) $BASIC 
rslider bounds(87, 73, 25, 25) channel("chrand") range(0, .9, .1, .5, 0.001) $BASIC 
}

;delay
image bounds(275,865,58,100) plant("Delay") $PLANT
{
;label bounds(2,2,30,10) text("Delay") $FONT
button bounds(8,2,40,17) channel("delon") text("Delay", "Delay") $BUTTONS
button bounds(10,24,27,15) text("Time") channel("mapdeltime") $MIDI
button bounds(37,24,9,15) text("") channel("mapdeltimeX") $MIDIX
rslider bounds(15,37,25,25) channel("deltime") range(0.001, 4.99, .08, 0.25, 0.0001) $BASIC
button bounds(10,60,27,15) text("FB") channel("mapdelFB") $MIDI
button bounds(37,60,9,15) text("") channel("mapdelFBX") $MIDIX
rslider bounds(15,73,25,25) channel("delFB") range(0, .99, .35, 1, 0.01) $BASIC
}

;reverbsc
image bounds(333,865,77,100) plant("Reverb") $PLANT
{
;label bounds(2,2,40,10) text("Reverb") $FONT
button bounds(16,2,45,17) channel("revon") text("Reverb", "Reverb") $BUTTONS
button bounds(7,28,30,15) text("Mix") channel("maprevmix") $MIDI
button bounds(37,28,9,15) text("") channel("maprevmixX") $MIDIX
rslider bounds(46,23,25,25) channel("revmix") range(0,1, .35, 1, 0.01) $BASIC
button bounds(7,51,30,15) text("Decay") channel("maprevFB") $MIDI
button bounds(37,51,9,15) text("") channel("maprevFBX") $MIDIX
rslider bounds(46,46,25,25) channel("revFB") range(0.01,0.999, .75, 1.75, 0.001) $BASIC
button bounds(7,74,30,15) text("LP") channel("maprevLP") $MIDI
button bounds(37,74,9,15) text("") channel("maprevLPX") $MIDIX
rslider bounds(46,69,25,25) channel("revLP") range(200, 20000, 12500, .5, 10) $BASIC
}

label bounds(465, 870, 60, 15) text("Presets") $FONT
combobox bounds(420, 885, 140, 15), channel("combobox"), populate("*.snaps")
filebutton bounds(565, 885, 40, 15), channel("preset"), text("Save"), mode("snapshot")
                                  
keyboard bounds(5,970,1015,75)
}
</Cabbage>

<CsoundSynthesizer>

<CsOptions>
-n -+rtmidi=NULL -M0 -m0d
</CsOptions>

<CsInstruments>

;sr is set by the host
ksmps = 64
nchnls = 2
0dbfs=1

massign	0,3
gichans		init	0
giReady		init	0
gSfilepath	init	""
gafinalL,gafinalR,gasrcL,gasrcR init 0
gkRecordingActiveFlag,gkFileRecorded init 0
giFFTSizes[]	array	32768, 16384, 8192, 4096, 2048, 1024, 512, 256, 128, 64, 32, 16, 8, 4	; an array is used to store FFT window sizes

; WINDOWING FUNCTIONS USED TO DYNAMICALLY SHAPE THE GRAINS
; NUM | INIT_TIME | SIZE | GEN_ROUTINE | PARTIAL_NUM | STRENGTH | PHASE
; GRAIN ENVELOPE WINDOW FUNCTION TABLES:
giwfn1	ftgen	0,  0, 131072,  20,   2, 1 									; HANNING
iTime	=	15000
giwfn2	ftgen	0,  0, 131072,  7,    0, iTime, 1, 131072-iTime, 0			; PERCUSSIVE - GEN08
giwfn3	ftgen	0,  0, 131072,  7,    0, 3072,   1, 128000,    0			; PERCUSSIVE - STRAIGHT SEGMENTS
giwfn4	ftgen	0,  0, 131072,  5,0.001, 3072,   1, 128000,0.001			; PERCUSSIVE - EXPONENTIAL SEGMENTS
giwfn5	ftgen	0,  0, 131072,  7,    0, 1536,   1, 128000,    1, 1536, 0	; GATE - WITH DE-CLICKING RAMP UP AND RAMP DOWN SEGMENTS
giwfn6	ftgen	0,  0, 131072,  7,    0, 131072-iTime, 1, iTime, 0			; REVERSE PERCUSSIVE - GEN08
giwfn7	ftgen	0,  0, 131072,  7,    0, 128000, 1, 3072,      0			; REVERSE PERCUSSIVE - STRAIGHT SEGMENTS
giwfn8	ftgen	0,  0, 131072,  5,0.001, 128000, 1, 3072,  0.001			; REVERSE PERCUSSIVE - EXPONENTIAL SEGMENTS

giTriangle	ftgen	0, 0, 4097,  20, 3


opcode	Grain3b,a,kkkkkkkkkkkiiikkiiikk
	kpch, kphs, kspd, kfreeze, kstart, kend, kshape, kfmd, kpmd, kdur, kdens, imaxovr, isfn, iwfn, kfrpow, kprpow , iseed, imode, iPhsMode, kDensRnd, kSizeRnd	xin

	if iPhsMode==1 then
	 kptr 		= 	kphs * (nsamp(isfn)/ftlen(isfn))	;MATHEMATICALLY REINTERPRET USER INPUTTED PHASE VALUE INTO A FORMAT THAT IS USABLE AS AN INPUT ARGUMENT  BY THE grain3 OPCODE

	elseif iPhsMode==2 then
	 kspd	*=	1-kfreeze
	 if kshape==1 then
	  kptr		phasor	(kspd*sr)/(nsamp(isfn)*(kend-kstart))
	  kpch		=	kpch-kspd
	 elseif kshape==2 then
	  kptr		oscili	1,(kspd*sr)/(nsamp(isfn)*(kend-kstart)*2),giTriangle
	  kptrPrev	init	0
	  kpch		=	kptr>kptrPrev?kpch-kspd:kpch+kspd
	  kptrPrev	=	kptr 
	 endif
	 kptr		*=	(kend-kstart)	 
	 kptr=kptr+kstart
	 kptr		mirror	kptr+kphs,0,1

	 kptr 		= 	kptr*(nsamp(isfn)/ftlen(isfn))	;MATHEMATICALLY REINTERPRET USER INPUTTED PHASE VALUE INTO A FORMAT THAT IS USABLE AS AN INPUT ARGUMENT  BY THE grain3 OPCODE

	endif

	kpch 		= 	(sr*kpch)/(ftlen(isfn))		;MATHEMATICALLY REINTERPRET USER INPUTTED PITCH RATIO VALUE INTO A FORMAT THAT IS USABLE AS AN INPUT ARGUMENT BY THE grain3 OPCODE - ftlen(x) FUNCTION RETURNS THE LENGTH OF A FUNCTION TABLE (no. x), REFER TO MANUAL FOR MORE INFO.	
	kfmd		=	(sr*(kfmd*kpch))/ftlen(isfn)

	ktrig		metro	kdens				;TRIGGERS IN SYNC WITH GRAIN GENERATION
	
	kDensRnd 	trandom ktrig,-kDensRnd,kDensRnd	;CREATE A RANDOM OFFSET FACTOR THAT WILL BE APPLIED TO FOR DENSITY
	kdens		*=	octave(kDensRnd)

	kSizeRnd 	trandom ktrig,-kSizeRnd,kSizeRnd	;CREATE A RANDOM OFFSET FACTOR THAT WILL BE APPLIED TO FOR DENSITY
	kdur		*=	octave(kSizeRnd)
	
	asig	grain3	kpch, kptr, kfmd, kpmd, kdur, kdens, imaxovr, isfn, iwfn, kfrpow, kprpow , iseed, imode
	xout	asig

endop

opcode	NextPowerOf2i,i,i
 iInVal	xin
 icount	=	1
 LOOP:
 if 2^icount>iInVal then
  goto DONE
 else
  icount	=	icount + 1
  goto LOOP
 endif
 DONE:
 	xout	2^icount
endop

opcode FileNameFromPath,S,S		; Extract a file name (as a string) from a full path (also as a string)
 Ssrc	xin				; Read in the file path string
 icnt	strlen	Ssrc			; Get the length of the file path string
 LOOP:					; Loop back to here when checking for a backslash
 iasc	strchar Ssrc, icnt		; Read ascii value of current letter for checking
 if iasc==92 igoto ESCAPE		; If it is a backslash, escape from loop
 loop_gt	icnt,1,0,LOOP		; Loop back and decrement counter which is also used as an index into the string
 ESCAPE:				; Escape point once the backslash has been found
 Sname	strsub Ssrc, icnt+1, -1		; Create a new string of just the file name
	xout	Sname			; Send it back to the caller instrument
endop

instr	1
 gkloop		chnget	"loop"
 gkPlayStop	chnget	"PlayStop"

 gkPhsMode	chnget	"PhsMode"
 gkPhsMode	init	1
 gklevel	chnget	"level"
 gklevel	port	gklevel,0.05
 gkdur		chnget	"dur"
 gkpch		chnget	"pch"
 
 gkwfn		chnget	"wfn"
 gkwfn		init	1
 gkspd		chnget	"spd"
 gkfreeze	chnget	"freeze"
 gkstart    chnget  "start"
 gkend      chnget  "end"
 gkshape	chnget	"shape"
 gkfmd		chnget	"fmd"
 gkpmd		chnget	"pmd"
 gkDensRnd	chnget	"DensRnd"
 gkSizeRnd	chnget	"SizeRnd"
 gkDensLFODep	chnget	"DensLFODep"
 gkAmpLFODep	chnget	"AmpLFODep"
 gkSizeLFODep	chnget	"SizeLFODep"
 gkLFORte	chnget	"LFORte"
 gkDualOnOff	chnget	"DualOnOff"
 gkDensRatio	chnget	"DensRatio"
 gkPtrDiff	chnget	"PtrDiff"
 gkTransDiff	chnget	"TransDiff"
 gkDelay	chnget	"Delay"
 gkDelay	port	gkDelay,0.1
 gkshiftsize chnget "shiftsize"
 gkwarpsize chnget "warpsize"
 gksmoothsize chnget "smoothsize"
 gkarpsize chnget "arpsize"
 gkTsize chnget "Tsize"
 
 if changed(gkPhsMode)==1 then
  if gkPhsMode==1 then
   	chnset	"visible(0)", "spdID"
   	chnset	"visible(0)", "freezeID"
   	chnset	"visible(0)", "rangeID"
   	chnset	"visible(0)", "shapeID"
   	chnset	"visible(0)", "shapelabelID"
  elseif gkPhsMode==2 then
   	chnset	"visible(1)", "spdID"
   	chnset	"visible(1)", "freezeID"
   	chnset	"visible(1)", "rangeID"
   	chnset	"visible(1)", "shapeID"
   	chnset	"visible(1)", "shapelabelID"
  endif
 endif
 
 gSfilepath	chnget	"filename"
 kNewFileTrg	changed	gSfilepath		; if a new file is loaded generate a trigger
 if kNewFileTrg==1 then				; if a new file has been loaded...
  event	"i",99,0,0				; call instrument to update sample storage function table 
 endif  

 /* START/STOP SOUNDING INSTRUMENT */
 ktrig	trigger	gkPlayStop,0.5,0
 schedkwhen	ktrig,0,0,2,0,-1

 /* MOUSE SCRUBBING */
 gkMOUSE_DOWN_RIGHT	chnget	"MOUSE_DOWN_RIGHT"	; Read in mouse left click status
 kStartScrub		trigger	gkMOUSE_DOWN_RIGHT,0.5,0
 if gkMOUSE_DOWN_RIGHT==1 then
  if kStartScrub==1 then 
   reinit RAMP_FUNC
  endif
  RAMP_FUNC:
  krampup	linseg	0,0.001,1
  rireturn
  kMOUSE_X	chnget	"MOUSE_X"
  kMOUSE_Y	chnget	"MOUSE_Y"
  kMOUSE_X	=	(kMOUSE_X - 5) / 930
  kMOUSE_Y	portk	1 - ((kMOUSE_Y - 5) / 370), krampup*0.05		; SOME SMOOTHING OF DENSITY CHANGES VIA THE MOUSE ENHANCES PERFORMANCE RESULTS. MAKE ANY ADJUSTMENTS WITH ADDITIONAL CONSIDERATION OF guirefresh VALUE 
  gkphs		limit	kMOUSE_X,0,1
  gkdens	limit	((kMOUSE_Y^3) * 499) + 1, 1, 500
  schedkwhen	kStartScrub,0,0,2,0,-1
 else
  gkphs		chnget	"phs"
  gkdens	chnget	"dens"
 endif

gkrecord	chnget	"record"
gkplay		chnget	"play"
kRecStart	trigger	gkrecord,0.5,0
  if kRecStart==1 && gkRecordingActiveFlag==0 then
    event	"i",600,0,-1
    gkRecordingActiveFlag	=	1
  endif
  kPlayStart	trigger	gkplay,0.5,0
  if kPlayStart==1 && gkFileRecorded==1 then
    event	"i",601,ksmps/sr,3600
  endif
endin

instr	2	; triggered by 'play/stop' button
 if gkPlayStop==0&&gkMOUSE_DOWN_RIGHT==0 then
  turnoff
 endif
 if giReady = 1 then						; i.e. if a file has been loaded
  
  /* ENVELOPE */
  iAttTim	chnget	"AttTim"				; read in widgets
  iRelTim	chnget	"RelTim"
  if iAttTim>0 then							; is amplitude envelope attack time is greater than zero...
   kenv	linsegr	0,iAttTim,1,iRelTim,0		; create an amplitude envelope with an attack, a sustain and a release segment (senses realtime release)
  else            
   kenv	linsegr	1,iRelTim,0					; create an amplitude envelope with a sustain and a release segment (senses realtime release)
  endif
  kenv	expcurve	kenv,8					; remap amplitude value with a more natural curve
  aenv	interp		kenv					; interpolate and create a-rate envelope

  kporttime	linseg	0,0.001,0.05			; portamento time function. (Rises quickly from zero to a held value.)

  kSwitch		changed	gkPhsMode, gkwfn
  if	kSwitch==1	then					; IF I-RATE VARIABLE CHANGE TRIGGER IS '1'...
  	reinit	START							; BEGIN A REINITIALISATION PASS FROM LABEL 'START'
  endif
  START:
  iwfn		=	giwfn1 + i(gkwfn) - 1
  
  /* LFO */
  kDensLFO	poscil	gkDensLFODep,gkLFORte
  kdens	=		gkdens * octave(kDensLFO)
  kAmpLFO	poscil	gkAmpLFODep*0.5,gkLFORte
  klevel	=		gklevel*(1-((kAmpLFO+(abs(gkAmpLFODep)*0.5))^2))
  kSizeLFO	poscil	gkSizeLFODep,gkLFORte
  kdur	=		gkdur * octave(kSizeLFO)
  
  iPhsMode	=	i(gkMOUSE_DOWN_RIGHT)==1?1:i(gkPhsMode)
  
  if gichans==1 then						; if mono...
   a1	Grain3b		gkpch, gkphs, gkspd, gkfreeze, gkstart, gkend, gkshape, gkfmd, gkpmd, kdur, kdens, 600, 1, iwfn, -2, -2 , rnd(1000), 8, iPhsMode, gkDensRnd, gkSizeRnd
   if gkDualOnOff==1 then
    a1b	Grain3b		gkpch+gkTransDiff, gkphs+gkPtrDiff, gkspd, gkfreeze, gkstart, gkend, gkshape, gkfmd, gkpmd, kdur, kdens*gkDensRatio, 600, 1, iwfn, -2, -2 , rnd(1000), 8, iPhsMode, gkDensRnd, gkSizeRnd
    if gkDelay>0 then
     a1b	vdelay	a1b,(gkDelay*1000)/kdens,5000
    endif
    a1	+=	a1b
   endif
 	outs	a1*aenv*klevel,a1*aenv*klevel		; send mono audio to both outputs 
  elseif gichans==2 then						; otherwise, if stereo...
   a1	Grain3b		gkpch, gkphs, gkspd, gkfreeze, gkstart, gkend, gkshape, gkfmd, gkpmd, kdur, kdens, 600, 1, iwfn, -2, -2 , rnd(1000), 8, iPhsMode, gkDensRnd, gkSizeRnd
   a2	Grain3b		gkpch, gkphs, gkspd, gkfreeze, gkstart, gkend, gkshape, gkfmd, gkpmd, kdur, kdens, 600, 2, iwfn, -2, -2 , rnd(1000), 8, iPhsMode, gkDensRnd, gkSizeRnd
   if gkDualOnOff==1 then
    a1b	Grain3b		gkpch+gkTransDiff, gkphs+gkPtrDiff, gkspd, gkfreeze, gkstart, gkend, gkshape, gkfmd, gkpmd, kdur, kdens*gkDensRatio, 600, 1, iwfn, -2, -2 , rnd(1000), 8, iPhsMode, gkDensRnd, gkSizeRnd
    a2b	Grain3b		gkpch+gkTransDiff, gkphs+gkPtrDiff, gkspd, gkfreeze, gkstart, gkend, gkshape, gkfmd, gkpmd, kdur, kdens*gkDensRatio, 600, 2, iwfn, -2, -2 , rnd(1000), 8, iPhsMode, gkDensRnd, gkSizeRnd
    if gkDelay>0 then
     a1b	vdelay	a1b,(gkDelay*1000)/kdens,5000
     a2b	vdelay	a2b,(gkDelay*1000)/kdens,5000
    endif
    a1	+=	a1b
    a2	+=	a2b
   endif
    krpan chnget "rpan"
    kprate chnget "prate"
    kprand chnget "prand"
    kpspread chnget "pspread"
    kLrate randh kprate*kprand,kprate,.1
    kLmetro=kprate+kLrate
    kLspread randomi 0,kpspread,kLmetro
    kRrate randh kprate*kprand,kprate,.25
    kRmetro=kprate+kRrate
    kRspread randomi 0,kpspread,kRmetro
    gafinalL=((a1*(1-kLspread))+(a2*kRspread))*aenv*klevel
    gafinalR=((a1*kLspread)+(a2*(1-kRspread)))*aenv*klevel
;    outs gafinalL,gafinalR
gasrcL=gafinalL+gasrcL
gasrcR=gafinalR+gasrcR
  endif
  rireturn

 endif
endin

instr	3
 icps	cpsmidi									; read in midi note data as cycles per second
 iamp	ampmidi	1								; read in midi velocity (as a value within the range 0 - 1)
 iAttTim	chnget	"AttTim"					; read in widgets
 iRelTim	chnget	"RelTim"
 iMidiRef	chnget	"MidiRef"
 iFrqRatio		=	icps/cpsmidinn(iMidiRef)	; derive playback speed from note played in relation to a reference note (MIDI note 60 / middle C)

 if giReady = 1 then							; i.e. if a file has been loaded
  iAttTim	chnget	"AttTim"					; read in widgets
  iRelTim	chnget	"RelTim"
  if iAttTim>0 then								; is amplitude envelope attack time is greater than zero...
   kenv	linsegr	0,iAttTim,1,iRelTim,0			; create an amplitude envelope with an attack, a sustain and a release segment (senses realtime release)
  else            
   kenv	linsegr	1,iRelTim,0						; create an amplitude envelope with a sustain and a release segment (senses realtime release)
  endif
  kenv	expcurve	kenv,8						; remap amplitude value with a more natural curve
  aenv	interp		kenv						; interpolate and create a-rate envelope
  kporttime	linseg	0,0.001,0.05				; portamento time function. (Rises quickly from zero to a held value.)

  kSwitch		changed	gkPhsMode, gkwfn
  if	kSwitch==1	then						; IF I-RATE VARIABLE CHANGE TRIGGER IS '1'...
  	reinit	START								; BEGIN A REINITIALISATION PASS FROM LABEL 'START'
  endif
  START:
  iwfn		=	giwfn1 + i(gkwfn) - 1

  /* LFO */
  kDensLFO	poscil	gkDensLFODep,gkLFORte
  kdens	=		gkdens * octave(kDensLFO)
  kAmpLFO	poscil	gkAmpLFODep*0.5,gkLFORte
  klevel	=		gklevel*(1-((kAmpLFO+(abs(gkAmpLFODep)*0.5))^2))
  kSizeLFO	poscil	gkSizeLFODep,gkLFORte
  kdur	=		gkdur * octave(kSizeLFO)

  iPhsMode	=	i(gkMOUSE_DOWN_RIGHT)==1?1:i(gkPhsMode)

  if gichans==1 then							; if mono...
   a1	Grain3b		iFrqRatio, gkphs, gkspd, gkfreeze, gkstart, gkend, gkshape, gkfmd, gkpmd, kdur, kdens, 600, 1, iwfn, -2, -2 , rnd(1000), 8, iPhsMode, gkDensRnd, gkSizeRnd
   if gkDualOnOff==1 then
    a1b	Grain3b		iFrqRatio+gkTransDiff, gkphs+gkPtrDiff, gkspd, gkfreeze, gkstart, gkend, gkshape, gkfmd, gkpmd, kdur, kdens*gkDensRatio, 600, 1, iwfn, -2, -2 , rnd(1000), 8, iPhsMode, gkDensRnd, gkSizeRnd
    if gkDelay>0 then
     a1b	vdelay	a1b,(gkDelay*1000)/kdens,5000
    endif
    a1	+=	a1b
   endif
 	outs	a1*aenv*klevel,a1*aenv*klevel		; send mono audio to both outputs 
  elseif gichans==2 then						; otherwise, if stereo...
   a1	Grain3b		iFrqRatio, gkphs, gkspd, gkfreeze, gkstart, gkend, gkshape, gkfmd, gkpmd, kdur, kdens, 600, 1, iwfn, -2, -2 , rnd(1000), 8, iPhsMode, gkDensRnd, gkSizeRnd
   a2	Grain3b		iFrqRatio, gkphs, gkspd, gkfreeze, gkstart, gkend, gkshape, gkfmd, gkpmd, kdur, kdens, 600, 2, iwfn, -2, -2 , rnd(1000), 8, iPhsMode, gkDensRnd, gkSizeRnd
   if gkDualOnOff==1 then
    a1b	Grain3b		iFrqRatio+gkTransDiff, gkphs+gkPtrDiff, gkspd, gkfreeze, gkstart, gkend, gkshape, gkfmd, gkpmd, kdur, kdens*gkDensRatio, 600, 1, iwfn, -2, -2 , rnd(1000), 8, iPhsMode, gkDensRnd, gkSizeRnd
    a2b	Grain3b		iFrqRatio+gkTransDiff, gkphs+gkPtrDiff, gkspd, gkfreeze, gkstart, gkend, gkshape, gkfmd, gkpmd, kdur, kdens*gkDensRatio, 600, 2, iwfn, -2, -2 , rnd(1000), 8, iPhsMode, gkDensRnd, gkSizeRnd
    if gkDelay>0 then
     a1b	vdelay	a1b,(gkDelay*1000)/kdens,5000
     a2b	vdelay	a2b,(gkDelay*1000)/kdens,5000
    endif
    a1	+=	a1b
    a2	+=	a2b
   endif
    kprate chnget "prate"
    kprand chnget "prand"
    kpspread chnget "pspread"
    kLrate randh kprate*kprand,kprate,.1
    kLmetro=kprate+kLrate
    kLspread randomi 0,kpspread,kLmetro
    kRrate randh kprate*kprand,kprate,.25
    kRmetro=kprate+kRrate
    kRspread randomi 0,kpspread,kRmetro
    gafinalL=((a1*(1-kLspread))+(a2*kRspread))*aenv*klevel
    gafinalR=((a1*kLspread)+(a2*(1-kRspread)))*aenv*klevel
;    outs gafinalL,gafinalR
gasrcL=gafinalL+gasrcL
gasrcR=gafinalR+gasrcR
  endif
  rireturn
 endif
endin

instr	99	; load sound file
 gichans	filenchnls	gSfilepath			; derive the number of channels (mono=1,stereo=2) in the sound file
 iFtlen		NextPowerOf2i	filelen:i(gSfilepath)*sr
 gitableL	ftgen	1,0,iFtlen,1,gSfilepath,0,0,1
 if gichans==2 then
  gitableR	ftgen	2,0,iFtlen,1,gSfilepath,0,0,2
 endif
 giReady 	=	1					; if no string has yet been loaded giReady will be zero
 Smessage sprintfk "file(%s)", gSfilepath			; print sound file to viewer
 chnset Smessage, "filer1"

 /* WRITE FILE NAME TO GUI */
 Sname FileNameFromPath	gSfilepath				; Call UDO to extract file name from the full path
 Smessage sprintfk "text(%s)",Sname
 chnset Smessage, "stringbox"
endin

instr 100   ;instrument to monitor MIDI controller numbers, creates global MIDI outputs that are evaluated by instr 110-300         
            ;also controls popup for latched LowPass and soundfile string management
gkstatus,gkchan,gkdata1,gkdata2 midiin
if changed(gkstatus)==1 then  
    if gkstatus==176 then
    chnset gkdata1,"CtrlNum" ;displays current controller number in GUI
    endif
endif
endin

instr 110 ;MIDI mapping for play panel
kmapspd chnget "mapspd"
kmapspdX chnget "mapspdX"
kmapdur chnget "mapdur"
kmapdurX chnget "mapdurX"
kmapdens chnget "mapdens"
kmapdensX chnget "mapdensX"
kmappch chnget "mappch"
kmappchX chnget "mappchX"

;monitor controller changes and assign
if changed(gkstatus)==1 then  
    if gkstatus==176 then
        if kmapspd == 1 then ;map control change knob to GUI, trigger instrument for respective knob
            turnoff2 111, 2, 0
            event "i", 111, 0, 999999999
        elseif kmapdur == 1 then
            turnoff2 112, 2, 0
            event "i", 112, 0, 999999999
        elseif kmapdens == 1 then
            turnoff2 113, 2, 0
            event "i", 113, 0, 999999999
        elseif kmappch == 1 then
            turnoff2 114, 2, 0
            event "i", 114, 0, 999999999
        endif
    endif
endif

if kmapspdX == 1 then ;release from controller
    turnoff2 111, 2, 0
elseif kmapdurX == 1 then
    turnoff2 112, 2, 0
elseif kmapdensX == 1 then
    turnoff2 113, 2, 0
elseif kmappchX == 1 then
    turnoff2 114, 2, 0
endif
endin

instr 111
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,-2,2 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "spd"
endin

instr 112
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,.01,5 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "dur"
endin

instr 113
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,.2,500 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "dens"
endin

instr 114
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,-2,2 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "pch"
endin

instr 120 ;MIDI mapping for manual control
kmapfmd chnget "mapfmd"
kmapfmdX chnget "mapfmdX"
kmappmd chnget "mappmd"
kmappmdX chnget "mappmdX"
kmapdensrnd chnget "mapdensrnd"
kmapdensrndX chnget "mapdensrndX"
kmapsizernd chnget "mapsizernd"
kmapsizerndX chnget "mapsizerndX"

;monitor controller changes and assign
if changed(gkstatus)==1 then  
    if gkstatus==176 then
        if kmapfmd == 1 then ;map control change knob to GUI, trigger instrument for respective knob
            turnoff2 121, 2, 0
            event "i", 121, 0, 999999999
        elseif kmappmd == 1 then
            turnoff2 122, 2, 0
            event "i", 122, 0, 999999999
        elseif kmapdensrnd == 1 then
            turnoff2 123, 2, 0
            event "i", 123, 0, 999999999
        elseif kmapsizernd == 1 then
            turnoff2 124, 2, 0
            event "i", 124, 0, 999999999
        endif
    endif
endif

if kmapfmdX == 1 then ;release from controller
    turnoff2 121, 2, 0
elseif kmappmdX == 1 then
    turnoff2 122, 2, 0
elseif kmapdensrndX == 1 then
    turnoff2 123, 2, 0
elseif kmapsizerndX == 1 then
    turnoff2 124, 2, 0
endif
endin

instr 121
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,0,8 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "fmd"
endin

instr 122
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,0,1 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "pmd"
endin

instr 123
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,0,2 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "DensRnd"
endin

instr 124
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,0,8 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "SizeRnd"
endin

instr 130 ;MIDI mapping for voice 2
kmapdensratio chnget "mapdensratio"
kmapdensratioX chnget "mapdensratioX"
kmapptrdiff chnget "mapptrdiff"
kmapptrdiffX chnget "mapptrdiffX"
kmaptransdiff chnget "maptransdiff"
kmaptransdiffX chnget "maptransdiffX"
kmapdelay chnget "mapdelay"
kmapdelayX chnget "mapdelayX"

;monitor controller changes and assign
if changed(gkstatus)==1 then  
    if gkstatus==176 then
        if kmapdensratio == 1 then ;map control change knob to GUI, trigger instrument for respective knob
            turnoff2 131, 2, 0
            event "i", 131, 0, 999999999
        elseif kmapptrdiff == 1 then
            turnoff2 132, 2, 0
            event "i", 132, 0, 999999999
        elseif kmaptransdiff == 1 then
            turnoff2 133, 2, 0
            event "i", 133, 0, 999999999
        elseif kmapdelay == 1 then
            turnoff2 134, 2, 0
            event "i", 134, 0, 999999999
        endif
    endif
endif

if kmapdensratioX == 1 then ;release from controller
    turnoff2 131, 2, 0
elseif kmapptrdiffX == 1 then
    turnoff2 132, 2, 0
elseif kmaptransdiffX == 1 then
    turnoff2 133, 2, 0
elseif kmapdelayX == 1 then
    turnoff2 134, 2, 0
endif
endin

instr 131
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,.5,2 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "DensRatio"
endin

instr 132
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,-1,1 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "PtrDiff"
endin

instr 133
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,-2,2 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "TransDiff"
endin

instr 134
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,0,1 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "Delay"
endin

instr 140 ;MIDI mapping for LFO
kmapdenslfodep chnget "mapdenslfodep"
kmapdenslfodepX chnget "mapdenslfodepX"
kmapamplfodep chnget "mapamplfodep"
kmapamplfodepX chnget "mapamplfodepX"
kmapsizelfodep chnget "mapsizelfodep"
kmapsizelfodepX chnget "mapsizelfodepX"
kmaplforte chnget "maplforte"
kmaplforteX chnget "maplforteX"

;monitor controller changes and assign
if changed(gkstatus)==1 then  
    if gkstatus==176 then
        if kmapdenslfodep == 1 then ;map control change knob to GUI, trigger instrument for respective knob
            turnoff2 141, 2, 0
            event "i", 141, 0, 999999999
        elseif kmapamplfodep == 1 then
            turnoff2 142, 2, 0
            event "i", 142, 0, 999999999
        elseif kmapsizelfodep == 1 then
            turnoff2 143, 2, 0
            event "i", 143, 0, 999999999
        elseif kmaplforte == 1 then
            turnoff2 144, 2, 0
            event "i", 144, 0, 999999999
        endif
    endif
endif

if kmapdenslfodepX == 1 then ;release from controller
    turnoff2 141, 2, 0
elseif kmapamplfodepX == 1 then
    turnoff2 142, 2, 0
elseif kmapsizelfodepX == 1 then
    turnoff2 143, 2, 0
elseif kmaplforteX == 1 then
    turnoff2 144, 2, 0
endif
endin

instr 141
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,-4,4 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "DensLFODep"
endin

instr 142
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,-1,1 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "AmpLFODep"
endin

instr 143
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,-2,2 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "SizeLFODep"
endin

instr 144
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,.01,8 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "LFORte"
endin

instr 150 ;MIDI mapping for attack, release, MIDI reference and level
kmapatttim chnget "mapatttim"
kmapatttimX chnget "mapatttimX"
kmapreltim chnget "mapreltim"
kmapreltimX chnget "mapreltimX"
kmapmidiref chnget "mapmidiref"
kmapmidirefX chnget "mapmidirefX"
kmaplevel chnget "maplevel"
kmaplevelX chnget "maplevelX"

;monitor controller changes and assign
if changed(gkstatus)==1 then  
    if gkstatus==176 then
        if kmapatttim == 1 then ;map control change knob to GUI, trigger instrument for respective knob
            turnoff2 151, 2, 0
            event "i", 151, 0, 999999999
        elseif kmapreltim == 1 then
            turnoff2 152, 2, 0
            event "i", 152, 0, 999999999
        elseif kmapmidiref == 1 then
            turnoff2 153, 2, 0
            event "i", 153, 0, 999999999
        elseif kmaplevel == 1 then
            turnoff2 154, 2, 0
            event "i", 154, 0, 999999999
        endif
    endif
endif

if kmapatttimX == 1 then ;release from controller
    turnoff2 151, 2, 0
elseif kmapreltimX == 1 then
    turnoff2 152, 2, 0
elseif kmapmidirefX == 1 then
    turnoff2 153, 2, 0
elseif kmaplevelX == 1 then
    turnoff2 154, 2, 0
endif
endin

instr 151
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,0,5 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "AttTim"
endin

instr 152
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,.01,5 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "RelTim"
endin

instr 153
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,0,127 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "MidiRef"
endin

instr 154
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,0,3 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "level"
endin

instr 160
;receive MIDI mapping buttons for Shift, Warp and Smooth
kmapshift chnget "mapshift"
kmapshiftX chnget "mapshiftX"
kmaplowest chnget "maplowest"
kmaplowestX chnget "maplowestX"
kmapshiftmix chnget "mapshiftmix"
kmapshiftmixX chnget "mapshiftmixX"
kmapenvscale chnget "mapenvscale"
kmapenvscaleX chnget "mapenvscaleX"
kmapenvshift chnget "mapenvshift"
kmapenvshiftX chnget "mapenvshiftX"
kmapenvlowest chnget "mapenvlowest"
kmapenvlowestX chnget "mapenvlowestX"
kmapwarpmix chnget "mapwarpmix"
kmapwarpmixX chnget "mapwarpmixX"
kmapampsmooth chnget "mapampsmooth"
kmapampsmoothX chnget "mapampsmoothX"
kmapfreqsmooth chnget "mapfreqsmooth"
kmapfreqsmoothX chnget "mapfreqsmoothX"
kmapsmoothmix chnget "mapsmoothmix"
kmapsmoothmixX chnget "mapsmoothmixX"

;monitor controller changes and assign
if changed(gkstatus)==1 then  
    if gkstatus==176 then
        if kmapshift == 1 then ;map control change knob to GUI, trigger instrument for respective knob
            turnoff2 161, 2, 0
            event "i", 161, 0, 999999999
        elseif kmaplowest == 1 then ;map control change knob to GUI, trigger instrument for respective knob
            turnoff2 162, 2, 0
            event "i", 162, 0, 999999999
        elseif kmapshiftmix == 1 then ;map control change knob to GUI, trigger instrument for respective knob
            turnoff2 163, 2, 0
            event "i", 163, 0, 999999999
        elseif kmapenvscale == 1 then ;map control change knob to GUI, trigger instrument for respective knob
            turnoff2 164, 2, 0
            event "i", 164, 0, 999999999
        elseif kmapenvshift == 1 then ;map control change knob to GUI, trigger instrument for respective knob
            turnoff2 165, 2, 0
            event "i", 165, 0, 999999999
        elseif kmapenvlowest == 1 then ;map control change knob to GUI, trigger instrument for respective knob
            turnoff2 166, 2, 0
            event "i", 166, 0, 999999999
        elseif kmapwarpmix == 1 then ;map control change knob to GUI, trigger instrument for respective knob
            turnoff2 167, 2, 0
            event "i", 167, 0, 999999999
        elseif kmapampsmooth == 1 then ;map control change knob to GUI, trigger instrument for respective knob
            turnoff2 168, 2, 0
            event "i", 168, 0, 999999999
        elseif kmapfreqsmooth == 1 then ;map control change knob to GUI, trigger instrument for respective knob
            turnoff2 169, 2, 0
            event "i", 169, 0, 999999999
        elseif kmapsmoothmix == 1 then ;map control change knob to GUI, trigger instrument for respective knob
            turnoff2 170, 2, 0
            event "i", 170, 0, 999999999
        endif
    endif
endif

;turn off MIDI controller GUI control for individual rsliders
if kmapshiftX == 1 then
    turnoff2 161, 2, 0
elseif kmaplowestX == 1 then
    turnoff2 162, 2, 0
elseif kmapshiftmixX == 1 then
    turnoff2 163, 2, 0
elseif kmapenvscaleX == 1 then
    turnoff2 164, 2, 0
elseif kmapenvshiftX == 1 then
    turnoff2 165, 2, 0
elseif kmapenvlowestX == 1 then
    turnoff2 166, 2, 0
elseif kmapwarpmixX == 1 then
    turnoff2 167, 2, 0
elseif kmapampsmoothX == 1 then
    turnoff2 168, 2, 0
elseif kmapfreqsmoothX == 1 then
    turnoff2 169, 2, 0
elseif kmapsmoothmixX == 1 then
    turnoff2 170, 2, 0
endif
endin

instr 161 ;MIDI mapping for Shift, Warp and Smooth
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,-1,1 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "shift"
endin

instr 162 ;MIDI mapping for Shift, Warp and Smooth
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,0,1 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "lowest"
endin

instr 163 ;MIDI mapping for Shift, Warp and Smooth
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,0,1 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "shiftmix"
endin

instr 164 ;MIDI mapping for Shift, Warp and Smooth
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,.1,10 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "envscale"
endin

instr 165 ;MIDI mapping for Shift, Warp and Smooth
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,-1,1 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "envshift"
endin

instr 166 ;MIDI mapping for Shift, Warp and Smooth
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,0,1 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "envlowest"
endin

instr 167 ;MIDI mapping for Shift, Warp and Smooth
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,0,1 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "warpmix"
endin

instr 168 ;MIDI mapping for Shift, Warp and Smooth
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,0,1 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "ampsmooth"
endin

instr 169 ;MIDI mapping for Shift, Warp and Smooth
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,0,1 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "freqsmooth"
endin

instr 170 ;MIDI mapping for Shift, Warp and Smooth
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,0,1 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "smoothmix"
endin

instr 180
;receive MIDI mapping buttons for Arp1
kmaparpbin1 chnget "maparpbin1"
kmaparpbin1X chnget "maparpbin1X"
kmaparpdepth1 chnget "maparpdepth1"
kmaparpdepth1X chnget "maparpdepth1X"
kmaparpgain1 chnget "maparpgain1"
kmaparpgain1X chnget "maparpgain1X"
kmaparpwidth1 chnget "maparpwidth1"
kmaparpwidth1X chnget "maparpwidth1X"
kmaparpspeed1 chnget "maparpspeed1"
kmaparpspeed1X chnget "maparpspeed1X"
kmaparpamp1 chnget "maparpamp1"
kmaparpamp1X chnget "maparpamp1X"
kmaparprand1 chnget "maparprand1"
kmaparprand1X chnget "maparprand1X"
kmaparpmix chnget "maparpmix"
kmaparpmixX chnget "maparpmixX"

;monitor controller changes and assign
if changed(gkstatus)==1 then  
    if gkstatus==176 then
        if kmaparpbin1 == 1 then ;map control change knob to GUI, trigger instrument for respective knob
            turnoff2 181, 2, 0
            event "i", 181, 0, 999999999
        elseif kmaparpdepth1 == 1 then ;map control change knob to GUI, trigger instrument for respective knob
            turnoff2 182, 2, 0
            event "i", 182, 0, 999999999
        elseif kmaparpgain1 == 1 then ;map control change knob to GUI, trigger instrument for respective knob
            turnoff2 183, 2, 0
            event "i", 183, 0, 999999999
        elseif kmaparpwidth1 == 1 then ;map control change knob to GUI, trigger instrument for respective knob
            turnoff2 184, 2, 0
            event "i", 184, 0, 999999999
        elseif kmaparpspeed1 == 1 then ;map control change knob to GUI, trigger instrument for respective knob
            turnoff2 185, 2, 0
            event "i", 185, 0, 999999999
        elseif kmaparpamp1 == 1 then ;map control change knob to GUI, trigger instrument for respective knob
            turnoff2 186, 2, 0
            event "i", 186, 0, 999999999
        elseif kmaparprand1 == 1 then ;map control change knob to GUI, trigger instrument for respective knob
            turnoff2 187, 2, 0
            event "i", 187, 0, 999999999
        elseif kmaparpmix == 1 then ;map control change knob to GUI, trigger instrument for respective knob
            turnoff2 188, 2, 0
            event "i", 188, 0, 999999999
        endif
    endif
endif

;turn off MIDI controller GUI control for individual rsliders
if kmaparpbin1X == 1 then
    turnoff2 181, 2, 0
elseif kmaparpdepth1X == 1 then
    turnoff2 182, 2, 0
elseif kmaparpgain1X == 1 then
    turnoff2 183, 2, 0
elseif kmaparpwidth1X == 1 then
    turnoff2 184, 2, 0
elseif kmaparpspeed1X == 1 then
    turnoff2 185, 2, 0
elseif kmaparpamp1X == 1 then
    turnoff2 186, 2, 0
elseif kmaparprand1X == 1 then
    turnoff2 187, 2, 0
elseif kmaparpmixX == 1 then
    turnoff2 188, 2, 0
endif
endin

instr 181 ;MIDI mapping for Arp1
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,0,1 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "arpbin1"
endin

instr 182 ;MIDI mapping for Arp1
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,0,100 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "arpdepth1"
endin

instr 183 ;MIDI mapping for Arp1
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,0,100 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "arpgain1"
endin

instr 184 ;MIDI mapping for Arp1
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,0,1 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "arpwidth1"
endin

instr 185 ;MIDI mapping for Arp1
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,0,100 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "arpspeed1"
endin

instr 186 ;MIDI mapping for Arp1
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,0,1 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "arpamp1"
endin

instr 187 ;MIDI mapping for Arp1
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,0,1 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "arprand1"
endin

instr 188 ;MIDI mapping for Arp1
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,0,1 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "arpmix"
endin

instr 190
;receive MIDI mapping buttons for Arp2
kmaparpbin2 chnget "maparpbin2"
kmaparpbin2X chnget "maparpbin2X"
kmaparpdepth2 chnget "maparpdepth2"
kmaparpdepth2X chnget "maparpdepth2X"
kmaparpgain2 chnget "maparpgain2"
kmaparpgain2X chnget "maparpgain2X"
kmaparpwidth2 chnget "maparpwidth2"
kmaparpwidth2X chnget "maparpwidth2X"
kmaparpspeed2 chnget "maparpspeed2"
kmaparpspeed2X chnget "maparpspeed2X"
kmaparpamp2 chnget "maparpamp2"
kmaparpamp2X chnget "maparpamp2X"
kmaparprand2 chnget "maparprand2"
kmaparprand2X chnget "maparprand2X"

;monitor controller changes and assign
if changed(gkstatus)==1 then  
    if gkstatus==176 then
        if kmaparpbin2 == 1 then ;map control change knob to GUI, trigger instrument for respective knob
            turnoff2 191, 2, 0
            event "i", 191, 0, 999999999
        elseif kmaparpdepth2 == 1 then ;map control change knob to GUI, trigger instrument for respective knob
            turnoff2 192, 2, 0
            event "i", 192, 0, 999999999
        elseif kmaparpgain2 == 1 then ;map control change knob to GUI, trigger instrument for respective knob
            turnoff2 193, 2, 0
            event "i", 193, 0, 999999999
        elseif kmaparpwidth2 == 1 then ;map control change knob to GUI, trigger instrument for respective knob
            turnoff2 194, 2, 0
            event "i", 194, 0, 999999999
        elseif kmaparpspeed2 == 1 then ;map control change knob to GUI, trigger instrument for respective knob
            turnoff2 195, 2, 0
            event "i", 195, 0, 999999999
        elseif kmaparpamp2 == 1 then ;map control change knob to GUI, trigger instrument for respective knob
            turnoff2 196, 2, 0
            event "i", 196, 0, 999999999
        elseif kmaparprand2 == 1 then ;map control change knob to GUI, trigger instrument for respective knob
            turnoff2 197, 2, 0
            event "i", 197, 0, 999999999            
        endif
    endif
endif

;turn off MIDI controller GUI control for individual rsliders
if kmaparpbin2X == 1 then
    turnoff2 191, 2, 0
elseif kmaparpdepth2X == 1 then
    turnoff2 192, 2, 0
elseif kmaparpgain2X == 1 then
    turnoff2 193, 2, 0
elseif kmaparpwidth2X == 1 then
    turnoff2 194, 2, 0
elseif kmaparpspeed2X == 1 then
    turnoff2 195, 2, 0
elseif kmaparpamp2X == 1 then
    turnoff2 196, 2, 0
elseif kmaparprand2X == 1 then
    turnoff2 197, 2, 0    
endif
endin

instr 191 ;MIDI mapping for Arp2
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,0,1 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "arpbin2"
endin

instr 192 ;MIDI mapping for Arp2
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,0,100 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "arpdepth2"
endin

instr 193 ;MIDI mapping for Arp2
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,0,100 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "arpgain2"
endin

instr 194 ;MIDI mapping for Arp2
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,0,1 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "arpwidth2"
endin

instr 195 ;MIDI mapping for Arp2
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,0,100 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "arpspeed2"
endin

instr 196 ;MIDI mapping for Arp1
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,0,1 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "arpamp2"
endin

instr 197 ;MIDI mapping for Arp1
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,0,1 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "arprand2"
endin

instr 200
;receive MIDI mapping buttons for Arp3
kmaparpbin3 chnget "maparpbin3"
kmaparpbin3X chnget "maparpbin3X"
kmaparpdepth3 chnget "maparpdepth3"
kmaparpdepth3X chnget "maparpdepth3X"
kmaparpgain3 chnget "maparpgain3"
kmaparpgain3X chnget "maparpgain3X"
kmaparpwidth3 chnget "maparpwidth3"
kmaparpwidth3X chnget "maparpwidth3X"
kmaparpspeed3 chnget "maparpspeed3"
kmaparpspeed3X chnget "maparpspeed3X"
kmaparpamp3 chnget "maparpamp3"
kmaparpamp3X chnget "maparpamp3X"
kmaparprand3 chnget "maparprand3"
kmaparprand3X chnget "maparprand3X"

;monitor controller changes and assign
if changed(gkstatus)==1 then  
    if gkstatus==176 then
        if kmaparpbin3 == 1 then ;map control change knob to GUI, trigger instrument for respective knob
            turnoff2 201, 2, 0
            event "i", 201, 0, 999999999
        elseif kmaparpdepth3 == 1 then ;map control change knob to GUI, trigger instrument for respective knob
            turnoff2 202, 2, 0
            event "i", 202, 0, 999999999
        elseif kmaparpgain3 == 1 then ;map control change knob to GUI, trigger instrument for respective knob
            turnoff2 203, 2, 0
            event "i", 203, 0, 999999999
        elseif kmaparpwidth3 == 1 then ;map control change knob to GUI, trigger instrument for respective knob
            turnoff2 204, 2, 0
            event "i", 204, 0, 999999999
        elseif kmaparpspeed3 == 1 then ;map control change knob to GUI, trigger instrument for respective knob
            turnoff2 205, 2, 0
            event "i", 205, 0, 999999999
        elseif kmaparpamp3 == 1 then ;map control change knob to GUI, trigger instrument for respective knob
            turnoff2 206, 2, 0
            event "i", 206, 0, 999999999
        elseif kmaparprand3 == 1 then ;map control change knob to GUI, trigger instrument for respective knob
            turnoff2 207, 2, 0
            event "i", 207, 0, 999999999
        endif
    endif
endif

;turn off MIDI controller GUI control for individual rsliders
if kmaparpbin3X == 1 then
    turnoff2 201, 2, 0
elseif kmaparpdepth3X == 1 then
    turnoff2 202, 2, 0
elseif kmaparpgain3X == 1 then
    turnoff2 203, 2, 0
elseif kmaparpwidth3X == 1 then
    turnoff2 204, 2, 0
elseif kmaparpspeed3X == 1 then
    turnoff2 205, 2, 0
elseif kmaparpamp3X == 1 then
    turnoff2 206, 2, 0
elseif kmaparprand3X == 1 then
    turnoff2 207, 2, 0
endif
endin

instr 201 ;MIDI mapping for Arp3
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,0,1 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "arpbin3"
endin

instr 202 ;MIDI mapping for Arp3
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,0,100 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "arpdepth3"
endin

instr 203 ;MIDI mapping for Arp3
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,0,100 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "arpgain3"
endin

instr 204 ;MIDI mapping for Arp3
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,0,1 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "arpwidth3"
endin

instr 205 ;MIDI mapping for Arp3
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,0,100 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "arpspeed3"
endin

instr 206 ;MIDI mapping for Arp1
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,0,1 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "arpamp3"
endin

instr 207 ;MIDI mapping for Arp1
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,0,1 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "arprand3"
endin

instr 210
;receive MIDI mapping buttons for Distortion
kmapdist chnget "mapdist"
kmapdistX chnget "mapdistX"
kmapdist2 chnget "mapdist2"
kmapdist2X chnget "mapdist2X"
kmapdist3 chnget "mapdist3"
kmapdist3X chnget "mapdist3X"

;monitor controller changes and assign
if changed(gkstatus)==1 then  
    if gkstatus==176 then
        if kmapdist == 1 then ;map control change knob to GUI, trigger instrument for respective knob
            turnoff2 211, 2, 0
            event "i", 211, 0, 999999999
        elseif kmapdist2 == 1 then ;map control change knob to GUI, trigger instrument for respective knob
            turnoff2 212, 2, 0
            event "i", 212, 0, 999999999
        elseif kmapdist3 == 1 then ;map control change knob to GUI, trigger instrument for respective knob
            turnoff2 213, 2, 0
            event "i", 213, 0, 999999999
        endif
    endif
endif

;turn off MIDI controller GUI control for individual rsliders
if kmapdistX == 1 then
    turnoff2 211, 2, 0
elseif kmapdist2X == 1 then
    turnoff2 212, 2, 0
elseif kmapdist3X == 1 then
    turnoff2 213, 2, 0
endif
endin

instr 211 ;MIDI mapping for Distortion
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,0,1 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "dist"
endin

instr 212 ;MIDI mapping for Distortion
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,0,.999 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "dist2"
endin

instr 213 ;MIDI mapping for Distortion
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,1,25 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "dist3"
endin

instr 230
;receive MIDI mapping buttons for Lowpass
kmapLPcps chnget "mapLPcps"
kmapLPcpsX chnget "mapLPcpsX"
kmapLPres chnget "mapLPres"
kmapLPresX chnget "mapLPresX"

;monitor controller changes and assign
if changed(gkstatus)==1 then  
    if gkstatus==176 then
        if kmapLPcps == 1 then ;map control change knob to GUI, trigger instrument for respective knob
            turnoff2 231, 2, 0
            event "i", 231, 0, 999999999
        elseif kmapLPres == 1 then ;map control change knob to GUI, trigger instrument for respective knob
            turnoff2 232, 2, 0
            event "i", 232, 0, 999999999
        endif
    endif
endif

;turn off MIDI controller GUI control for individual rsliders
if kmapLPcpsX == 1 then
    turnoff2 231, 2, 0
elseif kmapLPresX == 1 then
    turnoff2 232, 2, 0
endif
endin

instr 231 ;MIDI mapping for Lowpass
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,2,14 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "LPcps"
endin

instr 232 ;MIDI mapping for Lowpass
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,.01,.99 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "LPres"
endin

instr 240
;receive MIDI mapping buttons for Phaser
kmapphscps chnget "mapphscps"
kmapphscpsX chnget "mapphscpsX"
kmapphsFB chnget "mapphsFB"
kmapphsFBX chnget "mapphsFBX"
kmapphsQ chnget "mapphsQ"
kmapphsQX chnget "mapphsQX"


;monitor controller changes and assign
if changed(gkstatus)==1 then  
    if gkstatus==176 then
        if kmapphscps == 1 then ;map control change knob to GUI, trigger instrument for respective knob
            turnoff2 241, 2, 0
            event "i", 241, 0, 999999999
        elseif kmapphsFB == 1 then ;map control change knob to GUI, trigger instrument for respective knob
            turnoff2 242, 2, 0
            event "i", 242, 0, 999999999
        elseif kmapphsQ == 1 then ;map control change knob to GUI, trigger instrument for respective knob
            turnoff2 243, 2, 0
            event "i", 243, 0, 999999999
        endif
    endif
endif

;turn off MIDI controller GUI control for individual rsliders
if kmapphscpsX == 1 then
    turnoff2 241, 2, 0
elseif kmapphsFBX == 1 then
    turnoff2 242, 2, 0
elseif kmapphsQX == 1 then
    turnoff2 243, 2, 0
endif
endin

instr 241 ;MIDI mapping for Phaser
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,.01,100 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "phscps"
endin

instr 242 ;MIDI mapping for Phaser
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,0,1 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "phsFB"
endin

instr 243 ;MIDI mapping for Phaser
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,.01,100 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "phsQ"
endin

instr 250
;receive MIDI mapping buttons for Flanger
kmapflcps chnget "mapflcps"
kmapflcpsX chnget "mapflcpsX"
kmapflFB chnget "mapflFB"
kmapflFBX chnget "mapflFBX"

;monitor controller changes and assign
if changed(gkstatus)==1 then  
    if gkstatus==176 then
        if kmapflcps == 1 then ;map control change knob to GUI, trigger instrument for respective knob
            turnoff2 251, 2, 0
            event "i", 251, 0, 999999999
        elseif kmapflFB == 1 then ;map control change knob to GUI, trigger instrument for respective knob
            turnoff2 252, 2, 0
            event "i", 252, 0, 999999999
        endif
    endif
endif

;turn off MIDI controller GUI control for individual rsliders
if kmapflcpsX == 1 then
    turnoff2 251, 2, 0
elseif kmapflFBX == 1 then
    turnoff2 252, 2, 0
endif
endin

instr 251 ;MIDI mapping for Flanger
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,.01,100 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "flcps"
endin

instr 252 ;MIDI mapping for Flanger
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,0,.99 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "flFB"
endin

instr 260
;receive MIDI mapping buttons for Transposer
kmapharmmix chnget "mapharmmix"
kmapharmmixX chnget "mapharmmixX"
kmapharmon1C chnget "mapharmon1C"
kmapharmon1CX chnget "mapharmon1CX"
kmapharmon1F chnget "mapharmon1F"
kmapharmon1FX chnget "mapharmon1FX"
kmapharmon2C chnget "mapharmon2C"
kmapharmon2CX chnget "mapharmon2CX"
kmapharmon2F chnget "mapharmon2F"
kmapharmon2FX chnget "mapharmon2FX"
kmapharmon3C chnget "mapharmon3C"
kmapharmon3CX chnget "mapharmon3CX"
kmapharmon3F chnget "mapharmon3F"
kmapharmon3FX chnget "mapharmon3FX"
kmapharmon4C chnget "mapharmon4C"
kmapharmon4CX chnget "mapharmon4CX"
kmapharmon4F chnget "mapharmon4F"
kmapharmon4FX chnget "mapharmon4FX"
kmapharmon5C chnget "mapharmon5C"
kmapharmon5CX chnget "mapharmon5CX"
kmapharmon5F chnget "mapharmon5F"
kmapharmon5FX chnget "mapharmon5FX"
kmapharmon6C chnget "mapharmon6C"
kmapharmon6CX chnget "mapharmon6CX"
kmapharmon6F chnget "mapharmon6F"
kmapharmon6FX chnget "mapharmon6FX"
kmapTport chnget "mapTport"
kmapTportX chnget "mapTportX"

;monitor controller changes and assign
if changed(gkstatus)==1 then  
    if gkstatus==176 then
        if kmapharmmix == 1 then ;map control change knob to GUI, trigger instrument for respective knob
            turnoff2 261, 2, 0
            event "i", 261, 0, 999999999
        elseif kmapharmon1C == 1 then ;map control change knob to GUI, trigger instrument for respective knob
            turnoff2 262, 2, 0
            event "i", 262, 0, 999999999
        elseif kmapharmon1F == 1 then ;map control change knob to GUI, trigger instrument for respective knob
            turnoff2 263, 2, 0
            event "i", 263, 0, 999999999
        elseif kmapharmon2C == 1 then ;map control change knob to GUI, trigger instrument for respective knob
            turnoff2 264, 2, 0
            event "i", 264, 0, 999999999
        elseif kmapharmon2F == 1 then ;map control change knob to GUI, trigger instrument for respective knob
            turnoff2 265, 2, 0
            event "i", 265, 0, 999999999
        elseif kmapharmon3C == 1 then ;map control change knob to GUI, trigger instrument for respective knob
            turnoff2 266, 2, 0
            event "i", 266, 0, 999999999
        elseif kmapharmon3F == 1 then ;map control change knob to GUI, trigger instrument for respective knob
            turnoff2 267, 2, 0
            event "i", 267, 0, 999999999
        elseif kmapharmon4C == 1 then ;map control change knob to GUI, trigger instrument for respective knob
            turnoff2 268, 2, 0
            event "i", 268, 0, 999999999
        elseif kmapharmon4F == 1 then ;map control change knob to GUI, trigger instrument for respective knob
            turnoff2 269, 2, 0
            event "i", 269, 0, 999999999
        elseif kmapharmon5C == 1 then ;map control change knob to GUI, trigger instrument for respective knob
            turnoff2 270, 2, 0
            event "i", 270, 0, 999999999
        elseif kmapharmon5F == 1 then ;map control change knob to GUI, trigger instrument for respective knob
            turnoff2 271, 2, 0
            event "i", 271, 0, 999999999
        elseif kmapharmon6C == 1 then ;map control change knob to GUI, trigger instrument for respective knob
            turnoff2 272, 2, 0
            event "i", 272, 0, 999999999
        elseif kmapharmon6F == 1 then ;map control change knob to GUI, trigger instrument for respective knob
            turnoff2 273, 2, 0
            event "i", 273, 0, 999999999
        elseif kmapTport == 1 then ;map control change knob to GUI, trigger instrument for respective knob
            turnoff2 274, 2, 0
            event "i", 274, 0, 999999999
        endif
    endif
endif

;turn off MIDI controller GUI control for individual rsliders
if kmapharmmixX == 1 then
    turnoff2 261, 2, 0
elseif kmapharmon1CX == 1 then
    turnoff2 262, 2, 0
elseif kmapharmon1FX == 1 then
    turnoff2 263, 2, 0
elseif kmapharmon2CX == 1 then
    turnoff2 264, 2, 0
elseif kmapharmon2FX == 1 then
    turnoff2 265, 2, 0
elseif kmapharmon3CX == 1 then
    turnoff2 266, 2, 0
elseif kmapharmon3FX == 1 then
    turnoff2 267, 2, 0
elseif kmapharmon4CX == 1 then
    turnoff2 268, 2, 0
elseif kmapharmon4FX == 1 then
    turnoff2 269, 2, 0
elseif kmapharmon5CX == 1 then
    turnoff2 270, 2, 0
elseif kmapharmon5FX == 1 then
    turnoff2 271, 2, 0
elseif kmapharmon6CX == 1 then
    turnoff2 272, 2, 0
elseif kmapharmon6FX == 1 then
    turnoff2 273, 2, 0
elseif kmapTportX == 1 then
    turnoff2 274, 2, 0
endif
endin

instr 261 ;MIDI mapping for Transposer
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,0,1 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "harmmix"
endin

instr 262 ;MIDI mapping for Transposer
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,-24,24 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "harmon1C"
endin

instr 263 ;MIDI mapping for Transposer
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,-1,1 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "harmon1F"
endin

instr 264 ;MIDI mapping for Transposer
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,-24,24 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "harmon2C"
endin

instr 265 ;MIDI mapping for Transposer
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,-1,1 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "harmon2F"
endin

instr 266 ;MIDI mapping for Transposer
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,-24,24 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "harmon3C"
endin

instr 267 ;MIDI mapping for Transposer
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,-1,1 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "harmon3F"
endin

instr 268 ;MIDI mapping for Transposer
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,-24,24 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "harmon4C"
endin

instr 269 ;MIDI mapping for Transposer
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,-1,1 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "harmon4F"
endin

instr 270 ;MIDI mapping for Transposer
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,-24,24 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "harmon5C"
endin

instr 271 ;MIDI mapping for Transposer
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,-1,1 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "harmon5F"
endin

instr 272 ;MIDI mapping for Transposer
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,-24,24 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "harmon6C"
endin

instr 273 ;MIDI mapping for Transposer
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,-1,1 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "harmon6F"
endin

instr 274 ;MIDI mapping for Transposer
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,0,5 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "Tport"
endin

instr 280
;receive MIDI mapping buttons for Chorus
kmapchmix chnget "mapchmix"
kmapchmixX chnget "mapchmixX"
kmapchcps chnget "mapchcps"
kmapchcpsX chnget "mapchcpsX"
kmapchFB chnget "mapchFB"
kmapchFBX chnget "mapchFBX"
kmapchdepth chnget "mapchdepth"
kmapchdepthX chnget "mapchdepthX"
kmapchdel chnget "mapchdel"
kmapchdelX chnget "mapchdelX"
kmapchrand chnget "mapchrand"
kmapchrandX chnget "mapchrandX"

;monitor controller changes and assign
if changed(gkstatus)==1 then  
    if gkstatus==176 then
        if kmapchmix == 1 then ;map control change knob to GUI, trigger instrument for respective knob
            turnoff2 281, 2, 0
            event "i", 281, 0, 999999999
        elseif kmapchcps == 1 then ;map control change knob to GUI, trigger instrument for respective knob
            turnoff2 282, 2, 0
            event "i", 282, 0, 999999999
        elseif kmapchFB == 1 then ;map control change knob to GUI, trigger instrument for respective knob
            turnoff2 283, 2, 0
            event "i", 283, 0, 999999999
        elseif kmapchdepth == 1 then ;map control change knob to GUI, trigger instrument for respective knob
            turnoff2 284, 2, 0
            event "i", 284, 0, 999999999
        elseif kmapchdel == 1 then ;map control change knob to GUI, trigger instrument for respective knob
            turnoff2 285, 2, 0
            event "i", 285, 0, 999999999
        elseif kmapchrand == 1 then ;map control change knob to GUI, trigger instrument for respective knob
            turnoff2 286, 2, 0
            event "i", 286, 0, 999999999
        endif
    endif
endif

;turn off MIDI controller GUI control for individual rsliders
if kmapchmixX == 1 then
    turnoff2 281, 2, 0
elseif kmapchcpsX == 1 then
    turnoff2 282, 2, 0
elseif kmapchFBX == 1 then
    turnoff2 283, 2, 0
elseif kmapchdepthX == 1 then
    turnoff2 284, 2, 0
elseif kmapchdelX == 1 then
    turnoff2 285, 2, 0
elseif kmapchrandX == 1 then
    turnoff2 286, 2, 0
endif
endin

instr 281 ;MIDI mapping for Chorus
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,0,1 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "chmix"
endin

instr 282 ;MIDI mapping for Chorus
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,.01,25 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "chcps"
endin

instr 283 ;MIDI mapping for Chorus
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,0,.999 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "chFB"
endin

instr 284 ;MIDI mapping for Chorus
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,.1,50 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "chdepth"
endin

instr 285 ;MIDI mapping for Chorus
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,.1,50 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "chdel"
endin

instr 286 ;MIDI mapping for Chorus
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,0,.9 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "chrand"
endin

instr 290
;receive MIDI mapping buttons for Delay
kmapdeltime chnget "mapdeltime"
kmapdeltimeX chnget "mapdeltimeX"
kmapdelFB chnget "mapdelFB"
kmapdelFBX chnget "mapdelFBX"

;monitor controller changes and assign
if changed(gkstatus)==1 then  
    if gkstatus==176 then
        if kmapdeltime == 1 then ;map control change knob to GUI, trigger instrument for respective knob
            turnoff2 291, 2, 0
            event "i", 291, 0, 999999999
        elseif kmapdelFB == 1 then ;map control change knob to GUI, trigger instrument for respective knob
            turnoff2 292, 2, 0
            event "i", 292, 0, 999999999
        endif
    endif
endif

;turn off MIDI controller GUI control for individual rsliders
if kmapdeltimeX == 1 then
    turnoff2 291, 2, 0
elseif kmapdelFBX == 1 then
    turnoff2 292, 2, 0
endif
endin

instr 291 ;MIDI mapping for Delay
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,.001,4.99 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "deltime"
endin

instr 292 ;MIDI mapping for Delay
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,0,.99 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "delFB"
endin

instr 300
;receive MIDI mapping buttons for Reverb
kmaprevmix chnget "maprevmix"
kmaprevmixX chnget "maprevmixX"
kmaprevFB chnget "maprevFB"
kmaprevFBX chnget "maprevFBX"
kmaprevLP chnget "maprevLP"
kmaprevLPX chnget "maprevLPX"

;monitor controller changes and assign
if changed(gkstatus)==1 then  
    if gkstatus==176 then
        if kmaprevmix == 1 then ;map control change knob to GUI, trigger instrument for respective knob
            turnoff2 301, 2, 0
            event "i", 301, 0, 999999999
        elseif kmaprevFB == 1 then ;map control change knob to GUI, trigger instrument for respective knob
            turnoff2 302, 2, 0
            event "i", 302, 0, 999999999
        elseif kmaprevLP == 1 then ;map control change knob to GUI, trigger instrument for respective knob
            turnoff2 303, 2, 0
            event "i", 303, 0, 999999999
        endif
    endif
endif

;turn off MIDI controller GUI control for individual rsliders
if kmaprevmixX == 1 then
    turnoff2 301, 2, 0
elseif kmaprevFBX == 1 then
    turnoff2 302, 2, 0
elseif kmaprevLPX == 1 then
    turnoff2 303, 2, 0
endif
endin

instr 301 ;MIDI mapping for Reverb
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,0,1 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "revmix"
endin

instr 302 ;MIDI mapping for Reverb
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,.01,.999 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "revFB"
endin

instr 303 ;MIDI mapping for Reverb
ichan = i(gkchan)
idata1 = i(gkdata1)
ktr1 ctrl7 ichan,idata1,200,20000 ;map midi controller to GUI knob for ratios, scale values
chnset ktr1, "revLP"
endin

instr 400   ;FFT FX 
aL=gasrcL
aR=gasrcR

kshifton chnget "shifton"
if kshifton==0 goto noshift
  ktrig	changed	gkshiftsize
  if ktrig==1 then
   reinit shift
  endif
shift:
ishiftsize=giFFTSizes[i(gkshiftsize)]
kshift chnget "shift"
kshift portk kshift,.1
klowest chnget "lowest"
klowest portk klowest,.1
kshiftmix chnget "shiftmix"
fshiftanalL pvsanal aL,ishiftsize,ishiftsize*.25,ishiftsize,0
fshiftanalR pvsanal aR,ishiftsize,ishiftsize*.25,ishiftsize,0
fshiftL pvshift fshiftanalL,kshift*ishiftsize,klowest*ishiftsize
fshiftR pvshift fshiftanalR,kshift*ishiftsize,klowest*ishiftsize
ashiftL pvsynth fshiftL
ashiftR pvsynth fshiftR
aL=(ashiftL*kshiftmix)+(aL*(1-kshiftmix))
aR=(ashiftR*kshiftmix)+(aR*(1-kshiftmix))    

noshift:
kwarpon chnget "warpon"
if kwarpon==0 goto nowarp
  ktrig	changed	gkwarpsize
  if ktrig==1 then
   reinit warp
  endif
warp:
iwarpsize=giFFTSizes[i(gkwarpsize)]
kenvscale chnget "envscale"
kenvscale portk kenvscale,.1
kenvshift chnget "envshift"
kenvshift portk kenvshift,.1
kenvlowest chnget "envlowest"
kenvlowest portk kenvlowest,.1
kwarpmix chnget "warpmix"
fwarpanalL pvsanal aL,iwarpsize,iwarpsize*.25,iwarpsize,0
fwarpanalR pvsanal aR,iwarpsize,iwarpsize*.25,iwarpsize,0
fwarpL pvswarp fwarpanalL,kenvshift*iwarpsize,kenvscale*iwarpsize,kenvlowest*(sr*.5*iwarpsize)
fwarpR pvswarp fwarpanalR,kenvshift*iwarpsize,kenvscale*iwarpsize,kenvlowest*(sr*.5*iwarpsize)
awarpL pvsynth fwarpL
awarpR pvsynth fwarpR
aL=(awarpL*kwarpmix)+(aL*(1-kwarpmix))
aR=(awarpR*kwarpmix)+(aR*(1-kwarpmix))    

nowarp:
ksmoothon chnget "smoothon"
if ksmoothon==0 goto nosmooth
  ktrig	changed	gksmoothsize
  if ktrig==1 then
   reinit smooth
  endif
smooth:
ismoothsize=giFFTSizes[i(gksmoothsize)]
kampsmooth chnget "ampsmooth"
kfreqsmooth chnget "freqsmooth"
ksmoothmix chnget "smoothmix"
fsmoothanalL pvsanal aL,ismoothsize,ismoothsize*.25,ismoothsize,0
fsmoothanalR pvsanal aR,ismoothsize,ismoothsize*.25,ismoothsize,0
fsmoothL pvsmooth fsmoothanalL,1-kampsmooth,1-kfreqsmooth
fsmoothR pvsmooth fsmoothanalR,1-kampsmooth,1-kfreqsmooth
asmoothL pvsynth fsmoothL
asmoothR pvsynth fsmoothR
aL=(asmoothL*ksmoothmix)+(aL*(1-ksmoothmix))
aR=(asmoothR*ksmoothmix)+(aR*(1-ksmoothmix)) 

nosmooth:
karpon chnget "arpon"
if karpon==0 goto noarp
  ktrig	changed	gkarpsize
  if ktrig==1 then
   reinit arp
  endif
arp:
iarpsize=giFFTSizes[i(gkarpsize)]
karpbin1 chnget "arpbin1"
karpbin2 chnget "arpbin2"
karpbin3 chnget "arpbin3"
karpdepth1 chnget "karpdepth1"
karpdepth2 chnget "karpdepth2"
karpdepth3 chnget "karpdept3"
karpgain1 chnget "arpgain1"
karpgain2 chnget "arpgain2"
karpgain3 chnget "arpgain3"
karpwidth1 chnget "arpwidth1"
karpwidth2 chnget "arpwidth2"
karpwidth3 chnget "arpwidth3"
karpspeed1 chnget "arpspeed1"
karpspeed2 chnget "arpspeed2"
karpspeed3 chnget "arpspeed3"
karpamp1 chnget "arpamp1"
karpamp2 chnget "arpamp2"
karpamp3 chnget "arpamp3"
kshape1 chnget "arpshape1"
kshape2 chnget "arpshape2"
kshape3 chnget "arpshape3"
krand1 chnget "arprand1"
krand2 chnget "arprand2"
krand3 chnget "arprand3"
karpmix chnget "arpmix"

krandom1 randi karpspeed1*krand1,karpspeed1
kspeed1=karpspeed1+krandom1
krandom2 randi karpspeed2*krand2,karpspeed2
kspeed2=karpspeed2+krandom2
krandom3 randi karpspeed3*krand3,karpspeed3
kspeed3=karpspeed3+krandom3

if kshape1 = 1 then
kbin1a oscili karpwidth1,kspeed1,25
elseif kshape1 = 2 then 
kbin1a oscili karpwidth1,kspeed1,30
elseif kshape1 = 3 then 
kbin1a oscili karpwidth1,kspeed1,31
elseif kshape1 = 4 then 
kbin1a oscili karpwidth1,kspeed1,32
elseif kshape1 = 5 then 
kbin1a oscili karpwidth1,kspeed1,33
elseif kshape1 = 6 then 
kbin1a oscili karpwidth1,kspeed1,34
elseif kshape1 = 7 then 
kbin1a randh karpwidth1,kspeed1
elseif kshape1 = 8 then 
kbin1a randi karpwidth1,kspeed1
endif
kbin1=karpbin1+kbin1a

if kshape2 = 1 then
kbin2a oscili karpwidth2,kspeed2,25
elseif kshape2 = 2 then 
kbin2a oscili karpwidth2,kspeed2,30
elseif kshape2 = 3 then 
kbin2a oscili karpwidth2,kspeed2,31
elseif kshape2 = 4 then 
kbin2a oscili karpwidth2,kspeed2,32
elseif kshape2 = 5 then 
kbin2a oscili karpwidth2,kspeed2,33
elseif kshape2 = 6 then 
kbin2a oscili karpwidth2,kspeed2,34
elseif kshape2 = 7 then 
kbin2a randh karpwidth2,kspeed2
elseif kshape2 = 8 then 
kbin2a randi karpwidth2,kspeed2
endif
kbin2=karpbin2+kbin2a

if kshape3 = 1 then
kbin3a oscili karpwidth3,kspeed3,25
elseif kshape3 = 2 then 
kbin3a oscili karpwidth3,kspeed3,30
elseif kshape3 = 3 then 
kbin3a oscili karpwidth3,kspeed3,31
elseif kshape3 = 4 then 
kbin3a oscili karpwidth3,kspeed3,32
elseif kshape3 = 5 then 
kbin3a oscili karpwidth3,kspeed3,33
elseif kshape3 = 6 then 
kbin3a oscili karpwidth3,kspeed3,34
elseif kshape3 = 7 then 
kbin3a randh karpwidth3,kspeed3
elseif kshape3 = 8 then 
kbin3a randi karpwidth3,kspeed3
endif
kbin3=karpbin3+kbin3a

farpanalL pvsanal aL,iarpsize,iarpsize*.25,iarpsize,0
farpanalR pvsanal aR,iarpsize,iarpsize*.25,iarpsize,0
farp1L pvsarp farpanalL,kbin1,karpdepth1,karpgain1
farp1R pvsarp farpanalR,kbin1,karpdepth1,karpgain1
farp2L pvsarp farpanalL,kbin2,karpdepth2,karpgain2
farp2R pvsarp farpanalR,kbin2,karpdepth2,karpgain2
farp3L pvsarp farpanalL,kbin3,karpdepth3,karpgain3
farp3R pvsarp farpanalR,kbin3,karpdepth3,karpgain3
arp1L pvsynth farp1L
arp1R pvsynth farp1R
arp2L pvsynth farp2L
arp2R pvsynth farp2R
arp3L pvsynth farp3L
arp3R pvsynth farp3R
arpLsum=(arp1L*karpamp1)+(arp2L*karpamp2)+(arp3L*karpamp3)
arpRsum=(arp1R*karpamp1)+(arp2R*karpamp2)+(arp3R*karpamp3)
arpL balance arpLsum,aL
arpR balance arpRsum,aR
aL=(arpL*karpmix)+(aL*(1-karpmix))
aR=(arpR*karpmix)+(aR*(1-karpmix)) 

noarp:

gasrcL=aL;+gasrcL
gasrcR=aR;+gasrcR

endin

instr 500   ;global FX for EQ, distortion, LP, Phaser, Flanger, etc
aL=gasrcL
aR=gasrcR

;distortion
kdiston chnget "diston"
if kdiston==0 goto nodist
kdist chnget "distA"
kdist port kdist,.05
ad1L distort aL,kdist,26
ad1R distort aR,kdist,26
aL balance2 ad1L,aL
aR balance2 ad1R,aR

kdist2 chnget "distB"
kdist2=1-kdist2
kdist2 port kdist2,.05
ad2L powershape aL,kdist2
ad2R powershape aR,kdist2
aL balance2 ad2L,aL
aR balance2 ad2R,aR

kdist3 chnget "distC"
kdist3 port kdist3,.05
ad3L fold aL,kdist3
ad3R fold aR,kdist3
aL balance2 ad3L,aL
aR balance2 ad3R,aR

nodist:
;LP filter
kLPon chnget "LPon"
if kLPon==0 goto noLP
kLPsel chnget "LPsel"
kLPcps chnget "LPcps"
kLPcps port kLPcps,.1
kLPcfr chnget "LPcfr"
kLPcfd chnget "LPcfd"
kLPcfrand chnget "LPrand"
kLPres chnget "LPres"
kLPresr chnget "LPresr"
kLPresd chnget "LPresd"
kLPjit jitter kLPcfr*kLPcfrand,3,11
kLPcf oscili kLPcfd,kLPcfr+kLPjit,25
kLPQ oscili kLPresd,kLPresr,25
a1L,a2L,a3L,a4L mvclpf4 aL,cpsoct(kLPcf+kLPcps),kLPQ+kLPres
a1R,a2R,a3R,a4R mvclpf4 aR,cpsoct(kLPcf+kLPcps),kLPQ+kLPres
if kLPsel=1 then
alpL=a1L
alpR=a1R
elseif kLPsel=2 then
alpL=a2L
alpR=a2R
elseif kLPsel=3 then
alpL=a3L
alpR=a3R
elseif kLPsel=4 then
alpL=a4L
alpR=a4R
endif
aL balance alpL,aL
aR balance alpR,aR

noLP:
;4 Parallel Filters
kparon chnget "paron"
kfport chnget "fport"
if kparon==0 goto nopar
kparsel1 chnget "parsel1"
kparsel2 chnget "parsel2"
kparsel3 chnget "parsel3"
kparsel4 chnget "parsel4"
kparF1 chnget "parF1"
kparF2 chnget "parF2"
kparF3 chnget "parF3"
kparF4 chnget "parF4"
kparF1 portk kparF1,kfport
kparF2 portk kparF2,kfport
kparF3 portk kparF3,kfport
kparF4 portk kparF4,kfport
kparcfr1 chnget "parcfr1"
kparcfr2 chnget "parcfr2"
kparcfr3 chnget "parcfr3"
kparcfr4 chnget "parcfr4"
kparcfd1 chnget "parcfd1"
kparcfd2 chnget "parcfd2"
kparcfd3 chnget "parcfd3"
kparcfd4 chnget "parcfd4"
kparcfrnd1 chnget "parcfrnd1"
kparcfrnd2 chnget "parcfrnd2"
kparcfrnd3 chnget "parcfrnd3"
kparcfrnd4 chnget "parcfrnd4"
kparres1 chnget "parres1"
kparres2 chnget "parres2"
kparres3 chnget "parres3"
kparres4 chnget "parres4"
kparres1 portk kparres1,kfport
kparres2 portk kparres2,kfport
kparres3 portk kparres3,kfport
kparres4 portk kparres4,kfport
kparamp1 chnget "paramp1"
kparamp2 chnget "paramp2"
kparamp3 chnget "paramp3"
kparamp4 chnget "paramp4"
kparamp1 portk kparamp1,kfport
kparamp2 portk kparamp2,kfport
kparamp3 portk kparamp3,kfport
kparamp4 portk kparamp4,kfport
kparjit1 jitter kparcfr1*kparcfrnd1,2.5,8
kparjit2 jitter kparcfr2*kparcfrnd2,3.5,11
kparjit3 jitter kparcfr3*kparcfrnd3,3,10
kparjit4 jitter kparcfr4*kparcfrnd4,2.1,9
kpardev1 oscili kparcfd1,kparcfr1+kparjit1,25
kpardev2 oscili kparcfd2,kparcfr2+kparjit2,25
kpardev3 oscili kparcfd3,kparcfr3+kparjit3,25
kpardev4 oscili kparcfd4,kparcfr4+kparjit4,25
a1lpL,a1bpL,a1hpL zdf_2pole_mode aL,cpsoct(kparF1+kpardev1),kparres1
a1lpR,a1bpR,a1hpR zdf_2pole_mode aR,cpsoct(kparF1+kpardev1),kparres1
a2lpL,a2bpL,a2hpL zdf_2pole_mode aL,cpsoct(kparF2+kpardev2),kparres2
a2lpR,a2bpR,a2hpR zdf_2pole_mode aR,cpsoct(kparF2+kpardev2),kparres2
a3lpL,a3bpL,a3hpL zdf_2pole_mode aL,cpsoct(kparF3+kpardev3),kparres3
a3lpR,a3bpR,a3hpR zdf_2pole_mode aR,cpsoct(kparF3+kpardev3),kparres3
a4lpL,a4bpL,a4hpL zdf_2pole_mode aL,cpsoct(kparF4+kpardev4),kparres4
a4lpR,a4bpR,a4hpR zdf_2pole_mode aR,cpsoct(kparF4+kpardev4),kparres4
if kparsel1=1 then
apar1L=a1lpL
apar1R=a1lpR
elseif kparsel1=2 then
apar1L=a1bpL
apar1R=a1bpR
elseif kparsel1=3 then
apar1L=a1hpL
apar1R=a1hpR
elseif kparsel1=4 then
apar1L=0
apar1R=0
endif
if kparsel2=1 then
apar2L=a2lpL
apar2R=a2lpR
elseif kparsel2=2 then
apar2L=a2bpL
apar2R=a2bpR
elseif kparsel2=3 then
apar2L=a2hpL
apar2R=a2hpR
elseif kparsel2=4 then
apar2L=0
apar2R=0
endif
if kparsel3=1 then
apar3L=a3lpL
apar3R=a3lpR
elseif kparsel3=2 then
apar3L=a3bpL
apar3R=a3bpR
elseif kparsel3=3 then
apar3L=a3hpL
apar3R=a3hpR
elseif kparsel3=4 then
apar3L=0
apar3R=0
endif
if kparsel3=1 then
apar4L=a4lpL
apar4R=a4lpR
elseif kparsel4=2 then
apar4L=a4bpL
apar4R=a4bpR
elseif kparsel4=3 then
apar4L=a4hpL
apar4R=a4hpR
elseif kparsel4=4 then
apar4L=0
apar4R=0
endif
aparsumL=(apar1L*kparamp1)+(apar2L*kparamp2)+(apar3L*kparamp3)+(apar4L*kparamp4)
aparsumR=(apar1R*kparamp1)+(apar2R*kparamp2)+(apar3R*kparamp3)+(apar4R*kparamp4)
aL balance aparsumL,aL
aR balance aparsumR,aR

nopar:
;phaser
kphson chnget "phson"
if kphson==0 goto nophaser
kphscps chnget "phscps"
kphsQ chnget "phsQ"
kphsFB chnget "phsFB"
kphsdev oscil 25,kphscps,25
aLphs phaser2 aL,100+kphsdev,kphsQ,20,1,1,kphsFB
aRphs phaser2 aR,100+kphsdev,kphsQ,20,1,1,kphsFB
aL=(aL+aLphs)*.5
aR=(aR+aRphs)*.5

nophaser:
;flanger
kflon chnget "flon"
if kflon==0 goto noflanger
kflcps chnget "flcps"
kflFB chnget "flFB"
adel oscil .02,kflcps,27
aLfl flanger aL,adel*.05,kflFB,.1
aLfl clip aLfl,1,1
aRfl flanger aR,adel*.05,kflFB,.1
aRfl clip aRfl,1,1
aL=(aL+aLfl)*.5
aR=(aR+aRfl)*.5

noflanger:
;global transposer with 6 voices
kton chnget "ton"
if kton==0 goto noharmon
  ktrig	changed	gkTsize
  if ktrig==1 then
   reinit transposer
  endif
transposer:
iTsize=giFFTSizes[i(gkTsize)]

kharmon1 chnget "harmon1" ;on/off button
kharmon2 chnget "harmon2"
kharmon3 chnget "harmon3"
kharmon4 chnget "harmon4"
kharmon5 chnget "harmon5"
kharmon6 chnget "harmon6"
kharm1 port kharmon1,.05
kharm2 port kharmon2,.05
kharm3 port kharmon3,.05
kharm4 port kharmon4,.05
kharm5 port kharmon5,.05
kharm6 port kharmon6,.05

kTport chnget "Tport"
kT1C chnget "harmon1C"
kT1F chnget "harmon1F"
kT2C chnget "harmon2C"
kT2F chnget "harmon2F"
kT3C chnget "harmon3C"
kT3F chnget "harmon3F"
kT4C chnget "harmon4C"
kT4F chnget "harmon4F"
kT5C chnget "harmon5C"
kT5F chnget "harmon5F"
kT6C chnget "harmon6C"
kT6F chnget "harmon6F"
kT1C portk kT1C,kTport
kT1F portk kT1F,kTport
kT2C portk kT2C,kTport
kT2F portk kT2F,kTport
kT3C portk kT3C,kTport
kT3F portk kT3F,kTport
kT4C portk kT4C,kTport
kT4F portk kT4F,kTport
kT5C portk kT5C,kTport
kT5F portk kT5F,kTport
kT6C portk kT6C,kTport
kT6F portk kT6F,kTport

kshift1=kT1C+kT1F
kshift2=kT2C+kT2F
kshift3=kT3C+kT3F
kshift4=kT4C+kT4F
kshift5=kT5C+kT5F
kshift6=kT6C+kT6F
kharmmix chnget "harmmix"
ibase=cpsoct(8) ;middle C as base frequency

kratio1=cpsoct(8+(kshift1/12))/ibase ;Transposition ratio
kratio2=cpsoct(8+(kshift2/12))/ibase 
kratio3=cpsoct(8+(kshift3/12))/ibase 
kratio4=cpsoct(8+(kshift4/12))/ibase 
kratio5=cpsoct(8+(kshift5/12))/ibase 
kratio6=cpsoct(8+(kshift6/12))/ibase 

kjit1 jitter .002,.75,3.7
kjit2 jitter .001,.15,2.11
kjit3 jitter .0023,.1,1.4
kjit4 jitter .0025,.4,4.7
kjit5 jitter .0016,.6,3.8
kjit6 jitter .0019,.31,5.6

kratio1=kratio1+kjit1
kratio2=kratio2+kjit2
kratio3=kratio3+kjit3
kratio4=kratio4+kjit4
kratio5=kratio5+kjit5
kratio6=kratio6+kjit6

fftinL pvsanal aL,iTsize,iTsize*.25,iTsize,1
fftinR pvsanal aR,iTsize,iTsize*.25,iTsize,1

fftT1L pvscale fftinL,kratio1 ;pitch scaling and resynthesis
fftT1R pvscale fftinR,kratio1
aT1L pvsynth fftT1L
aT1R pvsynth fftT1R

fftT2L pvscale fftinL,kratio2
fftT2R pvscale fftinR,kratio2
aT2L pvsynth fftT2L
aT2R pvsynth fftT2R

fftT3L pvscale fftinL,kratio3
fftT3R pvscale fftinR,kratio3
aT3L pvsynth fftT3L
aT3R pvsynth fftT3R

fftT4L pvscale fftinL,kratio4
fftT4R pvscale fftinR,kratio4
aT4L pvsynth fftT4L
aT4R pvsynth fftT4R

fftT5L pvscale fftinL,kratio5
fftT5R pvscale fftinR,kratio5
aT5L pvsynth fftT5L
aT5R pvsynth fftT5R

fftT6L pvscale fftinL,kratio6
fftT6R pvscale fftinR,kratio6
aT6L pvsynth fftT6L
aT6R pvsynth fftT6R

asumL=(aT1L*kharm1)+(aT2L*kharm2)+(aT3L*kharm3)+(aT4L*kharm4)+(aT5L*kharm5)+(aT6L*kharm6)
asumR=(aT1R*kharm1)+(aT2R*kharm2)+(aT3R*kharm3)+(aT4R*kharm4)+(aT5R*kharm5)+(aT6R*kharm6)
aL=(aL*(1-kharmmix))+(asumL*kharmmix)
aR=(aR*(1-kharmmix))+(asumR*kharmmix)

noharmon:
gaFXL=aL
gaFXR=aR

endin


instr 502   ;global chorus
kchon chnget "chon"
kchmix chnget "chmix"
kchFB chnget "chFB"
kchdepth chnget "chdepth"
kchcps chnget "chcps"
kchrand chnget "chrand"
kchdel chnget "chdel"
if kchon==0 goto NOCHOR
achorusFBL init 0
achorusFBR init 0
afeedL=gaFXL+achorusFBL
afeedR=gaFXR+achorusFBR
kdepthrand=kchdepth*kchrand*.001
kchdel=kchdel*.001
kcpsrand=kchcps*kchrand
kchcps randomi kchcps-kcpsrand,kchcps+kcpsrand,kchcps
kchdepth randomi kchdepth-kdepthrand,kchdepth+kdepthrand,kchcps
kchdepth port kchdepth,.1
ardelL oscili kchdepth,kchcps,25
ardelR oscili kchdepth,kchcps,25,.25
ardel3 oscili kchdepth,kchcps,25,.5
ardel4 oscili kchdepth,kchcps,25,.75
ardelL=ardelL+(kchdepth*2.1+kchdel)
ardelR=ardelR+(kchdepth*2.1+kchdel)
ardel3=ardel3+(kchdepth*2.1+kchdel)
ardel4=ardel4+(kchdepth*2.1+kchdel)
amodL vdelay afeedL,ardelL,200
amodR vdelay afeedR,ardelR,200
amod3 vdelay afeedL*.33+afeedR*.66,ardel3,200
amod4 vdelay afeedR*.33+afeedL*.66,ardel4,200
achorusFBL=dcblock(((amodL*.6)+(amod4*.3)+(amod3*.15))*-kchFB)
achorusFBR=dcblock(((amodR*.6)+(amod3*.3)+(amod4*.15))*-kchFB)
gaChorL=((afeedL*.7+amodL)*kchmix)+((1-kchmix)*gaFXL)
gaChorR=((afeedR*.7+amodR)*kchmix)+((1-kchmix)*gaFXR)
goto LAST
NOCHOR:
gaChorL=gaFXL
gaChorR=gaFXR
LAST:
endin

instr 505   ;global FB delay FX
kdelon chnget "delon"
if kdelon==0 goto NODEL
aFBL init 0
aFBR init 0
kdel chnget "deltime"
kdel port kdel,.05
adel interp kdel*1000
kDelFB chnget "delFB"
aLdel vdelay gaChorL+aFBL,adel,5000
aRdel vdelay gaChorR+aFBR,adel,5000
aFBL=dcblock((aLdel*kDelFB)+gaChorL)
aFBR=dcblock((aRdel*kDelFB)+gaChorR)
gaDelL=aFBL
gaDelR=aFBR
goto LAST
NODEL:
gaDelL=gaChorL
gaDelR=gaChorR
LAST:
endin

instr 510   ;global reverb, always on
krevon chnget "revon"
if krevon==0 goto NOREV
krevmix chnget "revmix"
krevmix port krevmix,.05
krevFB chnget "revFB"
krevLP chnget "revLP"
aLrev,aRrev reverbsc gaDelL,gaDelR,krevFB,krevLP,sr,.075 ;don't need these to be global
gafinalL=(gaDelL*(1-krevmix))+(aLrev*krevmix)
gafinalR=(gaDelR*(1-krevmix))+(aRrev*krevmix)
goto FINALOUT
NOREV:
gafinalL=gaDelL
gafinalR=gaDelR
FINALOUT:
outs gafinalL,gafinalR
endin


instr 600	; record file
irecch chnget "recch"
ibit chnget "bit"
 print	p1
 if gkplay==1 then
  chnset	k(0),"record"
  turnoff
 endif
 gkFileRecorded		init	1
 itim     date
 Stim     dates     itim
 itim     date
 Stim     dates     itim
 Syear    strsub    Stim, 20, 24
 Smonth   strsub    Stim, 4, 7
 Sday     strsub    Stim, 8, 10
 iday     strtod    Sday
 Shor     strsub    Stim, 11, 13
 Smin     strsub    Stim, 14, 16
 Ssec     strsub    Stim, 17, 19
 Sfilnam  sprintf  "%s_%s_%02d_%s_%s_%s.wav", Syear, Smonth, iday, Shor,Smin, Ssec
 gSname   sprintf "Scrubber_%s", Sfilnam
 if gkrecord==1 then			; record
  if ibit=1 then
   imode=4
  elseif ibit=2 then
   imode=8
  elseif ibit=3 then
   imode=5
  endif
  if irecch=1 then
   fout gSname,imode,(gafinalL+gafinalR)*.7
  elseif irecch=2 then
   fout gSname,imode,gafinalL, gafinalR
  endif
 endif
 gkRecordingActiveFlag	=	1 - release()
endin

instr	601	; play file
 if gkplay==0 then
  turnoff
 endif 
   aL,aR	diskin2	gSname,1
  	outs	aL,aR
 iFileLen	filelen	gSname
 p3	=	iFileLen
 xtratim	0.1 
 krelease	release
 	chnset	1-krelease,"play"
endin

instr 750   ;clear global audio paths that include feedback (don't need to clear the global LFO outputs)
clear   gafinalL,gafinalR,gasrcL,gasrcR
endin

</CsInstruments>  

<CsScore>
f25 0 8193 10 1 ;sine for LFOs
f26 0 8193 7 -1 3500 -1 1193 1 3500 1   ;for distort opcode waveshaping
f27 0 16385 7 0 8192 1 8193 0   ;used to control flanger delay
f28 0 8193 10 1 0 .333 0 .2 0 .144 0 .1111 ;square wave
f29 0 8193 20 3 ;triangle envelope for harmonizer
f30 0 8193 7 0 2048 1 4097 -1 2048 0 ;triangle for arp
f31 0 8193 7 1 4096 1 0 -1 4097 -1 ;square for arp
f32 0 8193 7 0 8193 1 ;ramp up for arp
f33 0 8193 7 0 8193 -1 ;ramp down for arp
f34 0 8193 10 1 1 1 1 1 1 1 1 1 1 .9 .7 .5 .3 .1 ;pulse for arp

i1 0 999999
i75 0 999999
i100 0 999999
i110 0 999999
i120 0 999999
i130 0 999999
i140 0 999999
i150 0 999999
i160 0 999999
i180 0 999999
i190 0 999999
i200 0 999999
i210 0 999999
i230 0 999999
i240 0 999999
i250 0 999999
i260 0 999999
i280 0 999999
i290 0 999999
i300 0 999999
i400 0 999999
i500 0 999999
i502 0 999999
i505 0 999999
i510 0 999999
i750 0 999999
</CsScore>

</CsoundSynthesizer>
